/****************************************************************/
/* 								*/ 
/*   MODULE:  BDTest/hdlc100init.c				*/
/*   DATE:    98/12/01       					*/
/*   PURPOSE: hdlc(high level data link control) header		*/
/*                                                    		*/
/*--------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.     		*/
/*                                                 		*/
/*      programmed by jwCHOI & suCHOI				*/
/****************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002,2003		*/
/*			*/

#include "ks32c50.h"
#include "evm50100.h"

//#define HDLC_TX_ISR_DEBUG		// uncomment to debug Tx ISR
//#define HDLC_RX_ISR_DEBUG		// uncomment to debug Rx ISR
//#define USER_AREA
#define HDLC_1500

#define HDLCPORTNUM		    2	// HDLC ports count 
#define NonCache  	    0x4000000	// non cache bit

#define HDLC_MAX_TX_DES	           64	// Tx Buffer Descriptors count
#define HDLC_MAX_RX_DES	          512	// Rx Buffer Descriptors count

#ifdef HDLC_1500
#define HDLC_RX_BUF_LENGTH       1504   // Rx buffer length (bytes)
#define HDLC_TX_BUF_LENGTH       1504  	// Tx buffer length (bytes)
#else
#define HDLC_RX_BUF_LENGTH         64   // Rx buffer length (bytes)
#define HDLC_TX_BUF_LENGTH        128  	// Tx buffer length (bytes)
#endif 

// Tx alloc buffer size (words)
#define HDLC_TX_ALLOC_BUF_SIZE (HDLC_MAX_TX_DES*HDLC_TX_BUF_LENGTH/sizeof(U32))
// Rx alloc buffer size (words)
#define HDLC_RX_ALLOC_BUF_SIZE (HDLC_MAX_RX_DES*HDLC_RX_BUF_LENGTH/sizeof(U32))

#ifdef USER_AREA
// sizeof buffer to copy Rx Packets (words)
#define HDLC_USER_AREA_SIZE    (HDLC_MAX_RX_DES * HDLC_RX_BUF_LENGTH)	
#endif 							// USER_AREA

// 1500 Content HMFLR (Maximum Frame Lengt for DMA Mode) with CRC 
int HDLC_MAX_FRAME_LENGTH     =  1504;  

//////////////////////////////////////////////////////////////////////////////// 
// Tx/Rx Buffer descriptor structure (With Previous Descriptor) ////////////////
//////////////////////////////////////////////////////////////////////////////// 

typedef struct HDLCBD 			
{
 U32 BufferDataPtr;			// Data buffer pointer (+ownership bit) 
 U32 Reserved;				
 U32 StatusLength;
 struct HDLCBD * Next;
 struct HDLCBD * Prev;
} 
 sHDLCBD;


// HDLC Buffer Descriptors /////////////////////////////////////////////////////
sHDLCBD       sHDLCTxBD[HDLCPORTNUM][HDLC_MAX_TX_DES];	// Tx Buffer descriptors
sHDLCBD       sHDLCRxBD[HDLCPORTNUM][HDLC_MAX_RX_DES];	// Tx Buffer Descriptors

// alloc buffers ///////////////////////////////////////////////////////////////
U32  	      HDLCTxAllocBuf[HDLCPORTNUM][HDLC_TX_ALLOC_BUF_SIZE]; // Tx Data buffers
U32	      HDLCRxAllocBuf[HDLCPORTNUM][HDLC_RX_ALLOC_BUF_SIZE]; // Rx Data buffers

volatile U32  gHDLCTxStart[HDLCPORTNUM];  // Tx start BD pointer (moved by Tx ISR) 
volatile U32  gHDLCTxEnd[HDLCPORTNUM];	  // Tx end BD pointer (moved by Send)
volatile U32  gHDLCTxCount[HDLCPORTNUM]   // Used Tx BD count 	
              = {0, 0};
// 
volatile U32  gHDLCRxStart[HDLCPORTNUM];  // 
volatile U32  gHDLCRxEnd[HDLCPORTNUM];	  // Rx end BD pointer (moved by Rx ISR)
#ifndef USER_AREA
volatile U32  gHDLCRxCount[HDLCPORTNUM]   // Used Rx BD count 	
              = {0, 0};
#endif 							// not def USER_AREA


int 	      gHDLCCPUmode = 0;		  // 0 = DMA mode
volatile U32  gHDLCRxDone  = 0;		  // used in CPU Mode ISR

#ifdef USER_AREA
// Rx Queue ////////////////////////////////////////////////////////////////////
U32	      HDLCRxUserArea [HDLCPORTNUM][HDLC_USER_AREA_SIZE];
volatile U32 *HDLCRxUserRead [HDLCPORTNUM]={HDLCRxUserArea[0],HDLCRxUserArea[1]};
volatile U32 *HDLCRxUserWrite[HDLCPORTNUM]={HDLCRxUserArea[0],HDLCRxUserArea[1]};
volatile U32 *HDLCRxUserFirst[HDLCPORTNUM]={(U32 *) (-1), (U32 *) (-1) };
volatile int  HDLCRxUserCount[HDLCPORTNUM]={ 0, 0 };
#endif							// USER_AREA

// HDLC Statistics /////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////// 
// HDLC Tx Status Structure /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

typedef struct HdlcTxStatus 	
{
#ifdef HDLC_TX_ISR_DEBUG
 U32 TxISR;				// 
#endif 					// HDLC_TX_ISR_DEBUG
 U32 TxFC;				// Tx Frame Complete (TxFC)
 U32 TxFA;				// Tx FIFO Avail
 U32 TxSCTS;				// Tx CTS Store (TxSCTS)
 U32 TxU;				// Tx Underrun (TxU)
 U32 TxLBDNC;				// last BD Not completed
 U32 DMATxABT;				// Tx DMA Abort (DTxABT)
 U32 DMATxFD;				// Tx DMA Frame Done (DTxFD)		
 U32 DMATxNL;				// Tx DMA Null List (DTxNL)
 U32 DMATxNO;				// Tx DMA Not Owner (DTxNO)
} 
 HDLCTXSTATUS;

//////////////////////////////////////////////////////////////////////////////// 
// HDLC Rx Status Structure /////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////// 

typedef struct HdlcRxStatus 
{
#ifdef HDLC_RX_ISR_DEBUG
 U32 RxISR;
#endif 					// HDLC_RX_ISR_DEBUG
 U32 RxFA;				// Rx FIFO Avail (RxFA)
 U32 RxFD;				// Rx Flag Detect (RxFD)
 U32 RxSDCD;				// Rx DCD  store (RxSDCD)
 U32 RxFV;				// Rx Frame Valid (RxFV)
 U32 RxIDLE;				// Rx Idle
 U32 RxABT;				// Rx Abort
 U32 RxCRCE;				// Rx CRC Error	
 U32 RxNO;				// Rx Non Octet (RxNO)
 U32 RxOV;				// Rx Overrun (RxOV)
 U32 RxMOV;				// Rx memory overflow (RxMOV) 
 U32 RxIERR;				// Rx Inter Error (RxIERR)
 U32 DRxFD;				// Rx DMA FD (DRxFD)
 U32 DRxNL;				// Rx DMA NL (DRxNL)
 U32 DRxNO;				// Rx DMA NO (DRxNO)
} 
 HDLCRXSTATUS;
 

//////////////////////////////////////////////////////////////////////////////// 
// HDLC DPLL Status Structure //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

typedef struct HdlcDPLLStatus 
{
 U32 DPLLOM;
 U32 DPLLTM;
} 
 HDLCDPLLSTATUS;

// 
HDLCTXSTATUS   HDLCTxStatus[HDLCPORTNUM];   // Tx Statistics
HDLCRXSTATUS   HDLCRxStatus[HDLCPORTNUM];   // Rx Statistics
HDLCDPLLSTATUS HDLCDPLLStatus[HDLCPORTNUM]; // DPLL Statistics

////////////////////////////////////////////////////////////////////////////////
// HMODEA & HMODEB /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

U32  gHMODE = 			// HMODEA & HMODEB content 
// HMODE_MFinFIFO      |	// 1 = Multiple frame in FIFO 
 HMODE_RxLittle	     |		// Rx Little Endian
 HMODE_TxLittle	     |   	// Tx Little Endian

// Transmit Preamble Length ////////////////////////////////////////////////////
// HMODE_TxPL1			// 000 = 1 byte
// HMODE_TxPL2			// 001 = 2 bytes	
// HMODE_TxPL3		
// HMODE_TxPL4		
// HMODE_TxPL5		
// HMODE_TxPL6		
// HMODE_TxPL7		
// HMODE_TxPL8			// 111 = 8 bytes	

// Data Format /////////////////////////////////////////////////////////////////
 HMODE_DF_NRZ        | 
// HMODE_DF_NRZI		// 001 = NRZI (zero complement)
// HMODE_DF_FM0			// 010 = FM0
// HMODE_DF_FM1			// 011 = FM1
// HMODE_DF_Manchester		// 100 = Manchester

// DPLL Source /////////////////////////////////////////////////////////////////
// HMODE_DPLLCLK_TXC		// 000 = TXC
// HMODE_DPLLCLK_RXC		// 001 = RXC
// HMODE_DPLLCLK_MCLK		// 010 = MCLK
// HMODE_DPLLCLK_BRGOUT1	// 011 = BRGOUT1
// HMODE_DPLLCLK_BRGOUT2	// 100 = BRGOUT2

// BRG Source //////////////////////////////////////////////////////////////////
// HMODE_BRGCLK_RXC 		// 0  = RXC
 HMODE_BRGCLK_MCLK2  | 		// 1  = MCLK2

// TxCLK ///////////////////////////////////////////////////////////////////////
// HMODE_TxCLK_TXC    		// 000 = TXC
// HMODE_TxCLK_RXC     		// 001 = RXC
// HMODE_TxCLK_DPLLOUTT 	// 010 = DPLLOUTT
// MODE_TxCLK_BRGOUT1     	// 011 = BRGOUT1 
 HMODE_TxCLK_BRGOUT2 | 		// 100 = BRGOUT2

// HMODE_RxCLK_TXC		// 000 = TXC	
// HMODE_RxCLK_RXC		// 001 = RXC
// HMODE_RxCLK_DPLLOUTR		// 010 = DPLLOUTR
// HMODE_RxCLK_BRGOUT1		// 011 = BRGOUT1
 HMODE_RxCLK_BRGOUT2; 		// 100 = BRGOUT2	

// TxC Output Source ///////////////////////////////////////////////////////////
// HMODE_TXCOPS_TxCLK 	 	// 000 = TxCLK
// HMODE_TXCOPS_RxCLK  		// 001 = RxCLK	
// HMODE_TXCOPS_BRGOUT1  	// 010 = BRGOUT1
// HMODE_TXCOPS_BRGOUT2  	// 011 = BRGOUT2
// HMODE_TXCOPS_DPLLOUTT 	// 100 = DPLLOUTT
// HMODE_TXCOPS_DPLLOUTR 	// DPLLOUTR


////////////////////////////////////////////////////////////////////////////////
// HCONA & HCONB ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


volatile U32 gHCON = 		// HCONA & HCONB content 

// HCON_TxRS	   		// [0] Tx reset (TxRS)
// HCON_RxRS	     		// [1] Rx reset (RxRS)
// HCON_DTxRS	     		// [2] DMA Tx reset (DTxRS)
// HCON_DRxRS	     		// [3] DMA Rx reset (DRxRS)
// HCON_TxEN	 		// [4] Tx enable (TxEN)
// HCON_RxEN	    		// [5] Rx enable (RxEN)
// HCON_DTxEN	    		// [6] DMA Tx enable (DTxEN)
// HCON_DRxEN	    		// [7] DMA Rx enable (DRxEN)

// HCON_DPLLEN    		// [8] DPLL enable (DPLLEN)
 HCON_BRGEN     	|	// [9] BRG enable (BRGEN)

 HCON_Tx1WD		| 	// 1-word mode selected
// HCON_Tx4WD     	|	// 4-word mode selected

 HCON_Rx1WD		|	// 1-word mode selected
// HCON_Rx4WD     	| 	//  1 = 4-word mode selected

// Rx Allignment ///////////////////////////////////////////////////////////////
 HCON_RxWA0		|	// 00 = No Invalid bytes
// HCON_RxWA1          		// 01 = 1 invalid byte
// HCON_RxWA2          		// 10 = 2 invalid bytes
// HCON_RxWA3           	// 11 = 3 invalid bytes

HCON_DTxSTSK		|	// [14] DMA Tx stop or skip (DTxSTSK)
HCON_DRxSTSK		|      	// [15] DMA Rx stop or skip (DRxSTSK)

// HCON_DRxMADEC 		// [16] DMA Rx memory address decrement (DRxMADEC)
 HCON_TxFLAG    	|      	// [17] Tx flag idle (TxFLAG)
// HCON_TxSFLAG         	// [18] Tx single flag (TxSFLAG)
// HCON_TxLOOP          	// [19] Tx loop-back mode (TxLOOP)
// HCON_RxECHO          	// [20] Rx echo mode (RxECHO)
// HCON_TxABTEXT        	// [21] Tx abort extension (TxABTEXT)
// HCON_TxABT           	// [22] Tx abort (TxABT)
// HCON_TxPRMB          	// [23] Tx preamble (TxPRMB)

HCON_TxDTR     		;	// [24] Tx data terminal ready (TxDTR)
// HCON_RxDISCON        	// [25] Rx frame discontinue (RxDISCON)
// HCON_TxNOCRC         	// [26] Tx no CRC (TxNOCRC)
// HCON_RxNoCRC         	// [27] Rx no CRC (RxNOCRC)
// HCON_AutoEN;	          	// [28] Auto enable (AutoEN)


////////////////////////////////////////////////////////////////////////////////
// HINTENA HDLC Channel A Interrupt Enable /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

volatile U32 gHINTENA = 

// HINTEN_TxFCIE        	// [4] Tx frame complete interrupt enable
// HINTEN_TxFAIE        	// [5] Tx FIFO available to write interrupt enable
 HINTEN_TxSCTSIE    	|	// [7] CTS transition has occurred interrupt enable
 HINTEN_TxUIE         	|	// [8] Tx underrun has occurred interrupt enable
// HINTEN_RxFAIE        	// [9] Rx FIFO available to read interrupt enable
 HINTEN_RxFDIE     	|     	// [11]Rx flag detected interrupt enable
  HINTEN_RxSDCDIE   	|	// [13]DCD transition interrupt enable
// HINTEN_RxFVIE        	// [14]Rx frame valid interrupt enable
 HINTEN_RxIDLEIE  	|	// [15]Idle detected interrupt enable
 HINTEN_RxABTIE     	|	// [16]Abort detected interrupt enable
// HINTEN_RxCRCEIE  	    	// [17]CRC error frame interrupt enable
// HINTEN_RxNOIE        	// [18]Non-octet aligned frame interrupt enable
// HINTEN_RxOVIE        	// [19]Rx overrun interrupt enable

 HINTEN_RxMOVIE     	|	// [20]Rx memory overflow interrupt enable

 HINTEN_DTxABTIE    	| 	// [22]DMA Tx abort interrupt enable
 HINTEN_RxIERRIE    	|     	// [23]Rx internal error interrupt enable
 HINTEN_DRxFDIE     	|	// [24]DMA Rx frame done interrupt enable
 HINTEN_DRxNLIE     	|      	// [25]DMA Rx null list interrupt enable
 HINTEN_DRxNOIE      	|    	// [26]DMA Rx not owner interrupt enable

 HINTEN_DTxFDIE     	|  	// [27]DMA Tx frame done every transmitted frame interrupt enable
 HINTEN_DTxNLIE     	|	// [28]DMA Tx null list interrupt enable
 HINTEN_DTxNOIE  	;     	// [29]DMA Tx not owner interrupt enable
// HINTEN_DPLLOMIE	 	// [30]DPLL one clock missing interrupt enable
// HINTEN_DPLLTMIE;    		// [31]DPLL two clocks missing interrupt enable

////////////////////////////////////////////////////////////////////////////////
// HINTENB HDLC Channel B Interrupt Enable /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

volatile U32 gHINTENB = 

// HINTEN_TxFCIE        	// [4] Tx frame complete interrupt enable
// HINTEN_TxFAIE        	// [5] Tx FIFO available to write interrupt enable
 HINTEN_TxSCTSIE        |	// [7] CTS transition has occurred interrupt enable
 HINTEN_TxUIE         	|	// [8] Tx underrun has occurred interrupt enable
// HINTEN_RxFAIE        	// [9] Rx FIFO available to read interrupt enable
 HINTEN_RxFDIE   	|     	// [11]Rx flag detected interrupt enable
  HINTEN_RxSDCDIE 	|	// [13]DCD transition interrupt enable
// HINTEN_RxFVIE        	// [14]Rx frame valid interrupt enable
 HINTEN_RxIDLEIE  	|	// [15]Idle detected interrupt enable
 HINTEN_RxABTIE   	|	// [16]Abort detected interrupt enable
// HINTEN_RxCRCEIE  	    	// [17]CRC error frame interrupt enable
// HINTEN_RxNOIE        	// [18]Non-octet aligned frame interrupt enable
// HINTEN_RxOVIE        	// [19]Rx overrun interrupt enable
 HINTEN_RxMOVIE  	|	// [20]Rx memory overflow interrupt enable

 HINTEN_DTxABTIE 	|  	// [22]DMA Tx abort interrupt enable
 HINTEN_RxIERRIE 	|     	// [23]Rx internal error interrupt enable
 HINTEN_DRxFDIE  	|	// [24]DMA Rx frame done interrupt enable
 HINTEN_DRxNLIE  	|       // [25]DMA Rx null list interrupt enable
 HINTEN_DRxNOIE  	|       // [26]DMA Rx not owner interrupt enable

 HINTEN_DTxFDIE  	|  	// [27]DMA Tx frame done every transmitted frame interrupt enable
 HINTEN_DTxNLIE  	|	// [28]DMA Tx null list interrupt enable
 HINTEN_DTxNOIE  	;     	// [29]DMA Tx not owner interrupt enable
// HINTEN_DPLLOMIE;     	// [30]DPLL one clock missing interrupt enable
// HINTEN_DPLLTMIE;    		// [31]DPLL two clocks missing interrupt enable


////////////////////////////////////////////////////////////////////////////////
// View HDLC Global variables //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// return index in array BD, -1 = invalid pointer //////////////////////////////

static int HdlcVarIdx(U32 ptr, sHDLCBD * base)
{
 int index;
 index = (ptr & ~NonCache) - (((U32)(base)) & ~NonCache);
//Print("\nptr %x base %x index %d", ptr, (U32) base, index);
 if( (index % sizeof(sHDLCBD)) == 0 ) 
  {
   return(index / sizeof(sHDLCBD));
  }
 else
  {
   return(-1);
  } 
}


void HdlcVarView(void)
{
 int txstart[2];		// tx BD start index 
 int txend[2];			// tx BD end index  
 int txptr[2];			// tx BD pointer index 
 int rxstart[2];		// rx BD start index 	
 int rxend[2];			// rx BD end index
 int rxptr[2];			// rx BD pointer index 
 int cn;

 for(cn = 0; cn < 2; ++cn)
  {
   txstart[cn] = HdlcVarIdx(gHDLCTxStart[cn], & sHDLCTxBD[cn][0]);
   txend  [cn] = HdlcVarIdx(gHDLCTxEnd  [cn], & sHDLCTxBD[cn][0]);
   txptr  [cn] = HdlcVarIdx(HDMATXPTR   (cn), & sHDLCTxBD[cn][0]);
   rxstart[cn] = HdlcVarIdx(gHDLCRxStart[cn], & sHDLCRxBD[cn][0]);
   rxend  [cn] = HdlcVarIdx(gHDLCRxEnd  [cn], & sHDLCRxBD[cn][0]);
   rxptr  [cn] = HdlcVarIdx(HDMARXPTR   (cn), & sHDLCRxBD[cn][0]);
  }

#define PrintVarS(fmt,val0, val1  ) \
 Print(fmt "0x%08x %4s 0x%08x %4s", val0, "", val1, "")

#define PrintVarD(fmt,val0,val1, idx) \
 Print(fmt "0x%08x %4d 0x%08x %4d",val0,idx[0],val1,idx[1])

 while(1)
  {
   sHDLCBD * bd; 
   int num;
   U8 dir;
   U8 it;

   PrintVarS("\n[T] HDLCTxBD........",sHDLCTxBD      [0],sHDLCTxBD      [1]);
   PrintVarS("\n    HDLCTxBD Max Cnt",HDLC_MAX_TX_DES   ,HDLC_MAX_TX_DES   );
   PrintVarS("\n    HDLCTxAllocBuf..",HDLCTxAllocBuf [0],HDLCTxAllocBuf [1]);
   PrintVarD("\n    HDLCTxStart.....",gHDLCTxStart   [0],gHDLCTxStart   [1], txstart);
   PrintVarD("\n    HDLCTxEnd.......",gHDLCTxEnd     [0],gHDLCTxEnd     [1], txend);
   PrintVarS("\n    HDLCTxCount.....",gHDLCTxCount   [0],gHDLCTxCount   [1]);
   PrintVarD("\n    HDMATXPTR.......",HDMATXPTRA        ,HDMATXPTRB        , txptr);


   PrintVarS("\n[R] HDLCRxBD.......,",sHDLCRxBD      [0],sHDLCRxBD      [1]);
   PrintVarS("\n    HDLCRxBD Max Cnt",HDLC_MAX_RX_DES   ,HDLC_MAX_RX_DES   );
   PrintVarS("\n    HDLCRxAllocBuf..",HDLCRxAllocBuf [0],HDLCRxAllocBuf [1]);
   PrintVarD("\n    HDLCRxStart.....",gHDLCRxStart   [0],gHDLCRxStart   [1], rxstart);
   PrintVarD("\n    HDLCRxEnd.......",gHDLCRxEnd     [0],gHDLCRxEnd     [1], rxend); 

#ifndef USER_AREA
   PrintVarS("\n    HDLCRxCount.....",gHDLCRxCount[0],gHDLCRxCount[1]);
#endif 								// USER_AREA
   PrintVarD("\n    HDMARXPTR.......", HDMARXPTRA       , HDMARXPTRB        , rxptr);

#ifdef USER_AREA
   PrintVarS("\n    HDLCRxUserArea..",HDLCRxUserArea [0],HDLCRxUserArea [1]); 
   PrintVarS("\n    HDLCRxUserRead..",HDLCRxUserRead [0],HDLCRxUserRead [1]); 
   PrintVarS("\n    HDLCRxUserWrite.",HDLCRxUserWrite[0],HDLCRxUserWrite[1]);
   PrintVarS("\n    HDLCRxUserFirst.",HDLCRxUserFirst[0],HDLCRxUserFirst[1]);
   PrintVarS("\n    HDLCRxUserCount.",HDLCRxUserCount[0],HDLCRxUserCount[1]);
#endif 								// USER_AREA

   Print("\nHDLCCPUmode.....0x%08x", gHDLCCPUmode);
   Print("\ngHHDLCRxDone....0x%08x", gHDLCRxDone); 
   Print    ("\n[Q] Quit");
   Print    ("\n    Select Item");

// 
   dir = get_upper();			// read selected item from console
   if(dir == 'Q')
    {					// 
     return;
    } 

   if(dir != 'T' && dir != 'R')
    {					// not rx BD, tx BD	
     continue;
    }

// input channel number ////////////////////////////////////////////////////////
   Print("\nInput HDLC channel [0/1/Q]");
   cn = get_upper();
   if(cn == 'Q')     
    { 					// return from dump  
     return; 				
    } 
   else 
   if(cn >= '0' && cn <= '1')
    {
     cn = cn - '0';
    }
   else
    {
     continue;
    }  

// 
   bd = 0;					// dumped bd pointer

// input bd index //////////////////////////////////////////////////////////////
   Print("\nInput BD number           ");
   num = get_number(4,10); 
   if(dir == 'T') 
    {
     if(num < 0 || num >= HDLC_MAX_TX_DES)
      {
       Print("\nIncorrect Tx BD index %d. Must be from 0 to %d", 
        num, HDLC_MAX_TX_DES
       );
       continue;
      }
     bd = & sHDLCTxBD[cn][num];			// Tx BD, channel cn, index num 	
    }
   else
    {
     if(num < 0 || num >= HDLC_MAX_RX_DES)
      {
       Print("\nIncorrect Rx BD index %d. Must be from 0 to %d", 
        num, HDLC_MAX_RX_DES
       );
       continue;
      }
     bd = & sHDLCRxBD[cn][num];			// Rx BD, channel cn, index num
    }


   if(bd == 0)
    {
     continue;
    }

   while(1)
    {
// dump selected bd 
     Print("\nOwner....................%s", (bd->BufferDataPtr & HDLCTxBD_DMA) ? "1(DMA)" : "0(CPU)");
     Print("\nReserved...............0x%08x", bd->Reserved);
     if(dir == 'T')
      {
       Print("\n0   Preamble (P).........%s", ((bd->Reserved & HDLCTxBD_Preamble) ? "1" : "0")); 
       Print("\n1   TxNoCRC (N)..........%s", ((bd->Reserved & HDLCTxBD_TxNoCRCM) ? "1" : "0")); 
       Print("\n2   Little-Endian (E)....%s", ((bd->Reserved & HDLCTxBD_Little  ) ? "1" : "0")); 
       Print("\n3   Last (L).............%s", ((bd->Reserved & HDLCTxBD_Last    ) ? "1" : "0")); 
       Print("\n4   Decrement (D)........%s", ((bd->Reserved & HDLCTxBD_Dec     ) ? "1" : "0")); 
       Print("\n6:5 Widget Aligment (WA).");
       switch(bd->Reserved & HDLCTxBD_WA3)
        {
         case HDLCTxBD_WA0 : Print("00"); break;	// 00 = No invalid bytes
         case HDLCTxBD_WA1 : Print("01"); break;	// 01 = 1 Invalid bytes
         case HDLCTxBD_WA2 : Print("10"); break;	// 10 = 2 invalid bytes
         case HDLCTxBD_WA3 : Print("11"); break;	// 11 = 3 invalid bytes
         default: break;
        }
      }

// 
     Print("\nStatus Length..........0x%08x", bd->StatusLength);
     Print("\nLength.................0x%08x", bd->StatusLength & 0xFFFF);
     if(dir == 'T')
      {
       Print("\n26  Tx Comp............%s", ((bd->StatusLength & HDLCTxBD_TxComp) ? "1" : "0")); 
      }
     else
      {
// rx status
       U32 stat;
       stat = bd->StatusLength >> 16;
       Print("\n16 CD Lost (CD)..........%d", ((stat & HDLCRxBD_CD  )?1:0));
       Print("\n17 CRC Error (CE)........%d", ((stat & HDLCRxBD_CRC )?1:0));	    
       Print("\n18 Non-octet Aligned (NO)%d", ((stat & HDLCRxBD_NO  )?1:0));       
       Print("\n19 Overrun (OV)..........%d", ((stat & HDLCRxBD_OV  )?1:0));
       Print("\n20 DPLL Two Miss (DTM)...%d", ((stat & HDLCRxBD_DTM )?1:0));
       Print("\n21 Rx Abort (ABT)........%d", ((stat & HDLCRxBD_ABT )?1:0));
       Print("\n22 First In Frame (F)....%d", ((stat & HDLCRxBD_F   )?1:0));
       Print("\n23 Last In Frame (L).....%d", ((stat & HDLCRxBD_L   )?1:0));
       Print("\n24 FrameLen Violation....%d", ((stat & HDLCRxBD_FLV )?1:0));
      }
 
     Print("\nNext BD................0x%08x", bd->Next);
      {
       int next;			
       if(dir == 'T')
        {	
         next = HdlcVarIdx((U32)bd->Next, sHDLCTxBD[cn]);	// next Tx BD  
        }
       else
        {
         next = HdlcVarIdx((U32)bd->Next, sHDLCRxBD[cn]);	// next Rx BD
        }
       Print("(%d)", next);
      }

     Print("\nPres any key to select next BD, Q - Quit");
     it = get_upper();
     if(it == 'Q' || it == 'q')
      {
       return;
      } 

     num++;  					// move to next BD 
     if(dir == 'T') 
      {
       if(num >= HDLC_MAX_TX_DES)
        {					
         num = 0;
        }
       Print("\nHDLC%c Tx BD %d", 'A' + cn, num);
       bd = & sHDLCTxBD[cn][num];
      }
     else
      {
       if(num >= HDLC_MAX_RX_DES)
        {
         num = 0; 
        }
       Print("\nHDLC%c Rx BD %d", 'A' + cn, num);
       bd = & sHDLCRxBD[cn][num];
      }
    } 
  }
}


////////////////////////////////////////////////////////////////////////////////
// View HDLC Error Report //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void HdlcErrorRpt(void)
{
 Print("\n                   %8s %8s","A","B");
 Print( "                    %8s %8s","A","B");
 Print("\nHDLC Tx Status");

////////////////////////////////////////////////////////////////////////////////
// HDLC Tx Statistic ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifdef HDLC_TX_ISR_DEBUG
 Print("\nTx ISR             %8d %8d",
   HDLCTxStatus[0].TxISR    , 
   HDLCTxStatus[1].TxISR
  );
#endif 							// HDLC_TX_ISR_DEBUG

// Tx Complete (in DMA mode) ///////////////////////////////////////////////////
 Print("\nTx Complete        %8d %8d",		/* 1 */
   HDLCTxStatus[0].TxFC    , 
   HDLCTxStatus[1].TxFC
  );

// Tx FIFO Avail ///////////////////////////////////////////////////////////////

 Print( " Tx FIFO Avail      %8d %8d",		/* 1 */
   HDLCTxStatus[0].TxFA    ,
   HDLCTxStatus[1].TxFA
  );

// Tx CTS Store ////////////////////////////////////////////////////////////////
 Print("\nTx CTS Store       %8d %8d",		/* 2 */
   HDLCTxStatus[0].TxSCTS   	,
   HDLCTxStatus[1].TxSCTS
  );

// Tx Underrun /////////////////////////////////////////////////////////////////
 Print( " Tx Underrun        %8d %8d",		/* 2 */
   HDLCTxStatus[0].TxU		,
   HDLCTxStatus[1].TxU
  );

// Tx DMA Frame Done ///////////////////////////////////////////////////////////
 Print("\nTx DMA Frame Done  %8d %8d",		/* 3 */
   HDLCTxStatus[0].DMATxFD	   ,
   HDLCTxStatus[1].DMATxFD
  );

// Tx DMA Abort ////////////////////////////////////////////////////////////////
 Print( " Tx DMA Abort       %8d %8d",		/* 3 */
   HDLCTxStatus[0].DMATxABT  	   ,
   HDLCTxStatus[1].DMATxABT
  );

// Tx DMA Null List ////////////////////////////////////////////////////////////
 Print("\nTx DMA Null List   %8d %8d",		/* 4 */
   HDLCTxStatus[0].DMATxNL	   ,
   HDLCTxStatus[1].DMATxNL
  );

// Tx DMA Not Owner ////////////////////////////////////////////////////////////
 Print( " Tx DMA Not Owner   %8d %8d",		/* 4 */
   HDLCTxStatus[0].DMATxNO	   ,
   HDLCTxStatus[1].DMATxNO
  );

// ISR error Last Tx BD not completed  //////////////////////////////////////////
 Print("\nLast BD Not Comp   %8d %8d",	
   HDLCTxStatus[0].TxLBDNC	   ,
   HDLCTxStatus[1].TxLBDNC
  );


////////////////////////////////////////////////////////////////////////////////
// HDLC Rx Statistic ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifdef HDLC_RX_ISR_DEBUG
 Print("\nRx ISR             %8d %8d",		/* 1 */
   HDLCRxStatus[0].RxISR    , 
   HDLCRxStatus[1].RxISR
  );
#endif 							// HDLC_RX_ISR_DEBUG

   Print("\nHDLC Rx Status")	;		/* 5 */
// Rx FIFO Avail ///////////////////////////////////////////////////////////////
 Print("\nRx FIFO Avail      %8d %8d",		/* 6 */
   HDLCRxStatus[0].RxFA	   ,
   HDLCRxStatus[1].RxFA
  );  

// Rx Flag Detect //////////////////////////////////////////////////////////////
 Print( " Rx Flag Detect     %8d %8d",		/* 6 */
   HDLCRxStatus[0].RxFD 	,		
   HDLCRxStatus[1].RxFD		
  );  

// Rx DCD  store ///////////////////////////////////////////////////////////////
 Print("\nRx DCD  store      %8d %8d",		/* 7 */
   HDLCRxStatus[0].RxSDCD       ,
   HDLCRxStatus[1].RxSDCD
  );  
 
// Rx LFrame Valid /////////////////////////////////////////////////////////////
 Print( " Rx Frame Valid     %8d %8d",		/* 7 */
   HDLCRxStatus[0].RxFV 	,
   HDLCRxStatus[1].RxFV
  );  

// Rx Idle /////////////////////////////////////////////////////////////////////
 Print("\nRx Idle            %8d %8d",		/* 8 */
   HDLCRxStatus[0].RxIDLE        ,
   HDLCRxStatus[1].RxIDLE
  );  

// Rx Abort ////////////////////////////////////////////////////////////////////
 Print( " Rx Abort           %8d %8d",		/* 8 */
   HDLCRxStatus[0].RxABT        ,
   HDLCRxStatus[1].RxABT
  );  

// Rx CRC Error	////////////////////////////////////////////////////////////////
 Print("\nRx CRC Error       %8d %8d",		/* 9 */
   HDLCRxStatus[0].RxCRCE       ,
   HDLCRxStatus[1].RxCRCE
  );  

// Rx Non Octet ////////////////////////////////////////////////////////////////
 Print( " Rx Non Octet       %8d %8d",		/* 9 */
   HDLCRxStatus[0].RxNO     	,
   HDLCRxStatus[1].RxNO
  );  

// Rx Overrun //////////////////////////////////////////////////////////////////
 Print("\nRx Overrun         %8d %8d",		/* 10 */
   HDLCRxStatus[0].RxOV      	,
   HDLCRxStatus[1].RxOV
  );  
 
// Rx RxMOV ////////////////////////////////////////////////////////////////////
 Print( " Rx RxMOV           %8d %8d",		/* 10 */
   HDLCRxStatus[0].RxMOV       ,
   HDLCRxStatus[1].RxMOV
  );  

// Rx Inter Error //////////////////////////////////////////////////////////////
 Print("\nRx Inter Error     %8d %8d",		/* 11 */
   HDLCRxStatus[0].RxIERR 	   ,
   HDLCRxStatus[1].RxIERR
  );  

// Rx DMA NL ///////////////////////////////////////////////////////////////////
 Print( " Rx DMA NL          %8d %8d",		/* 11 */
   HDLCRxStatus[0].DRxNL           ,
   HDLCRxStatus[1].DRxNL
  );  

// Rx DMA FD ///////////////////////////////////////////////////////////////////
 Print("\nRx DMA FD          %8d %8d",		/* 12 */
   HDLCRxStatus[0].DRxFD           , 
   HDLCRxStatus[1].DRxFD
  );  
 
 
// Rx DMA NO ///////////////////////////////////////////////////////////////////
 Print( " Rx DMA NO          %8d %8d",		/* 12 */
   HDLCRxStatus[0].DRxNO        ,
   HDLCRxStatus[1].DRxNO
  );  

// 
 Print("\nDPLL One ClkMiss   %8d %8d",		/* 13 */
   HDLCDPLLStatus[0].DPLLOM	,
   HDLCDPLLStatus[1].DPLLOM
  );

 Print( " DPLL Two ClkMiss   %8d %8d",		/* 13 */
   HDLCDPLLStatus[0].DPLLTM,
   HDLCDPLLStatus[1].DPLLTM
  );
}

////////////////////////////////////////////////////////////////////////////////
// Clear HDLC Statistics ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HdlcClrErrReport()
{	
 U8 cn;
 for(cn = 0; cn <= 1; ++cn)
  {
#ifdef HDLC_TX_ISR_DEBUG
   HDLCTxStatus[cn].TxISR     = 0;	
#endif 					// HDLC_RX_ISR_DEBUG

   HDLCTxStatus[cn].TxFC     = 0;	// Tx Frame Complete
   HDLCTxStatus[cn].TxFA     = 0;	// Tx FIFO Avail
   HDLCTxStatus[cn].TxSCTS   = 0;
   HDLCTxStatus[cn].TxU      = 0;	// 
   HDLCTxStatus[cn].TxLBDNC  = 0;	// Last BD not completed 
   HDLCTxStatus[cn].DMATxABT = 0;	// Tx DMA Abort
   HDLCTxStatus[cn].DMATxFD  = 0;	// Tx DMA Frame Done
   HDLCTxStatus[cn].DMATxNL  = 0;	// Tx DMA Null List
   HDLCTxStatus[cn].DMATxNO  = 0;	// Tx DMA Not Owner

// HDLC Rx Status //////////////////////////////////////////////////////////////
#ifdef HDLC_TX_ISR_DEBUG
   HDLCRxStatus[cn].RxISR    = 0;	// 
#endif 					// HDLC_TX_ISR_DEBUG

   HDLCRxStatus[cn].RxFA     = 0;	// Rx FIFO Avail
   HDLCRxStatus[cn].RxFD     = 0;	// Rx Flag Detect
   HDLCRxStatus[cn].RxSDCD   = 0;	// Rx DCD  store
   HDLCRxStatus[cn].RxFV     = 0;	// Rx Frame Valid
   HDLCRxStatus[cn].RxIDLE   = 0;	// Rx Idle
   HDLCRxStatus[cn].RxABT    = 0;	// Rx Abort
   HDLCRxStatus[cn].RxCRCE   = 0;	// Rx CRC Error
   HDLCRxStatus[cn].RxNO     = 0;	// Rx Non Octet
   HDLCRxStatus[cn].RxOV     = 0;	// Rx Overrun
   HDLCRxStatus[cn].RxMOV    = 0;	// Rx RxMOV
   HDLCRxStatus[cn].RxIERR   = 0;	// Rx Inter Error
   HDLCRxStatus[cn].DRxFD    = 0;	// Rx DMA FD
   HDLCRxStatus[cn].DRxNL    = 0;	// Rx DMA NL
   HDLCRxStatus[cn].DRxNO    = 0;	// Rx DMA NO

// HDLC DPLL Status ////////////////////////////////////////////////////////////

   HDLCDPLLStatus[cn].DPLLOM = 0;	// one clock miss
   HDLCDPLLStatus[cn].DPLLTM = 0;	// two clock miss
  }
}


////////////////////////////////////////////////////////////////////////////////
// GetBaudRate /////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/* [1:0]    Time constant value for CNT2 				      */
/*           00 = divide by 1 						      */
/*           01 = divide by 16 						      */
/*           10 = divide by 32 						      */
/* [3:2]    Time constant value for CNT1 				      */	
/*           00 = divide by 1 						      */
/*           01 = divide by 16 						      */
/* [15:4]   Time constant value for CNT0 				      */
/* 								              */
/* BRGOUT1 = (MCLK2 or RXC) / (CNT0 + 1) / (16 ** CNT1 ) 	              */
/* BRGOUT2 = BRGOUT1 / (1 or 16 or 32 according to CNT2 value of the HBRGTC)  */


static U32 GetBaudRate(U32 WantedBR)
{
 U32 DIF;				// minimal baudrate difference
 U32 CNT0;				// selected counter CNT0 
 U32 CNT1;				// selected counter CNT1
 U32 CNT2;				// selected counter CNT2
 U32 i;
	
//	
 DIF = 0xFFFFFFFF;			// start minimal baudrate difference

 for(i = 0; i < 6; ++i)
  {
   U32 scal0;
   U32 scal1;
   U32 scal2;
   U32 dif;
   U32 br;
   U32 j;

   switch(i) 
    {
     case 0 : scal1 =  1 ; scal2 =  1 ; break;
     case 1 : scal1 =  1 ; scal2 = 16 ; break;
     case 2 : scal1 =  1 ; scal2 = 32 ; break;
     case 3 : scal1 = 16 ; scal2 =  1 ; break;
     case 4 : scal1 = 16 ; scal2 = 16 ; break;
     case 5 : scal1 = 16 ; scal2 = 32 ; break;
     default : ;
    }

   scal0 = MCLK2 / WantedBR / scal1 / scal2;	
   for(j = 0; j < 2; j++, scal0++)		// 
    {						// scal0 = CNT0 + 1
     if(scal0 >= 1 && scal0 <= 4096)		// CNT0 >= 0 && CNT0 <= 4095
      {					
       br  = MCLK2 / scal0 / scal1 / scal2;	// calculated baudrate 
       if(br > WantedBR) {dif = br - WantedBR;} // baurate differense 
       else              {dif = WantedBR - br;} //

       if(dif < DIF)
        {					// less then minimal dif 
         DIF =  dif;				// reset minimal difference 
         CNT0 = scal0 - 1;			// set counter CNT0 
         CNT1 = scal1 / 16;			// set counter CNT1
         CNT2 = scal2 / 16;			// set counter CNT2
        }
      }
    }
  } 

// 
 return((CNT0 <<4) | (CNT1<<2) | CNT2 );
}


////////////////////////////////////////////////////////////////////////////////
// Tx Buffer Descriptor generation /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// cn          = channel num (0,1)
// TxBufLength = transmit buffer length 
//               0 = default length 0x120
// NullPoint   = null pointer entry
//               0 = circular BD list
//               else prev[NullPoint]->next = 0


void TxBD_init(int cn, int NullPoint)
{
 sHDLCBD * bd;				// current buffer descriptor
 sHDLCBD * start;			// start transmit buffer descriptor 
 sHDLCBD * prev;			// previous buffer descriptor
 U32 allocptr;				// alloc position 
 U32 alloccnt; 				// bytes in buffer 
 U32 i;
 int TxBufLength;

// align word boundary /////////////////////////////////////////////////////////
 TxBufLength = ((HDLC_TX_BUF_LENGTH + 3) >> 2) << 2;
//
 if(NullPoint <= 0 || NullPoint > HDLC_MAX_TX_DES)
  {
   NullPoint = HDLC_MAX_TX_DES;
  }

 HDMATXPTR(cn)    = 			// HDMATXPTR
 gHDLCTxStart[cn] = 			// Tx BD start position 
 gHDLCTxEnd  [cn] = 			// Tx BD end 
  (U32) sHDLCTxBD[cn] | 
        NonCache;
 gHDLCTxCount[cn] = 0;			// Tx BD count with DMA owner bit

// 
 bd = (sHDLCBD *) gHDLCTxStart[cn];	// current buffer descriptor
 prev  = (sHDLCBD *)NULL;		// previous buffer descriptor
 start = bd;				// save start buffer descriptor

// buffer allocation pointer + non cacheble area
 allocptr  = (U32) HDLCTxAllocBuf[cn] | NonCache;	

// sizeof of alloc buffer, words
 alloccnt  = HDLC_TX_ALLOC_BUF_SIZE * sizeof(U32);	    
 for(i = 0; i < HDLC_MAX_TX_DES; ++i)
  {
   if(TxBufLength > alloccnt)
    {
     break;				// allocation error
    }

// set fields off current buffer descriptor ////////////////////////////////////
   bd->BufferDataPtr = 			// frame data pointer 
    (U32) allocptr & 			// allocation pointer 
      HDLCTxBD_CPU;			// clear DMA bit 
// 
   bd->Reserved      = 0x0; 				
   bd->StatusLength  = TxBufLength;   	// tx length 
   bd->Next          = 0; 		// pointer to next BD
   bd->Prev          = prev; 		// pointer to previous BD	

// 
   if(prev != (sHDLCBD *) NULL)
    {
     prev->Next = bd;		
    }

   if(i == NullPoint) 
    {
     break;
    }

// 
   prev = bd; 				// reset previous BD  
   bd++;				// move to next 
		
   allocptr += TxBufLength; 		// allocation position 
   alloccnt -= TxBufLength;		// remain bytes in alloc buffer
  }

//
 if(NullPoint == HDLC_MAX_TX_DES)
  {
// circular BD list /////////////////////////////////////////////////////////////

   bd--;			// Last BD 		
   bd->Next = start;		// pointer to start descriptor 
   start->Prev = bd; 		
  }
}

////////////////////////////////////////////////////////////////////////////////
// Rx Buffe Descriptors Init ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void RxBD_init(int cn) 
{
 sHDLCBD * bd;			// current BD 
 sHDLCBD * prev;		// previous BD	
 sHDLCBD * start;		// start buffer descriptor
 U32 allocptr;			// allocation position 
 U32 alloccnt;			// remain bytes in allocation buffer 
 U32 i;

 HDMARXPTR(cn) =		// current Rx BD pointer
 gHDLCRxStart[cn] = 		// start BD pointer  
 gHDLCRxEnd[cn] =  		// end BD pointer 
  (U32) sHDLCRxBD[cn] | 	
        NonCache;

#ifndef USER_AREA
 gHDLCRxCount[cn] = 0;		// Rx BD count 
#endif 								// USER_AREA

 bd    = (sHDLCBD *)gHDLCRxStart[cn];
 start =  bd;
 prev  = (sHDLCBD *)NULL ;
 
 allocptr = (U32) HDLCRxAllocBuf[cn] | NonCache;   // start allocation position 
 alloccnt = HDLC_RX_ALLOC_BUF_SIZE * sizeof(U32);  // bytes in alloc buffer
 
 for(i = 0; i < HDLC_MAX_RX_DES; ++i)
  {
   if(HDLC_RX_BUF_LENGTH > alloccnt)
    {
// out off alloc buffer //////////////////////////////////////////////////////// 
     break;
    }

// set fields off current buffer descriptor ////////////////////////////////////    
   bd->BufferDataPtr = 			// buffer data pointer 
    allocptr   | 			// current allocation position 
    HDLCRxBD_DMA;			// owner HDMA

// 
   bd->Reserved      = 0x0;   
   bd->StatusLength  = 0x0;   		// status & length 
   bd->Next          = NULL;  		// next BD 
   bd->Prev          = prev;  		// previous BD

   if(prev != (sHDLCBD *) NULL)
    {
     prev->Next = bd;		
    }
    
   prev = bd;				// reset previous BD 
   bd++;				// move to next buffer descriptor 

// 		
   allocptr += HDLC_RX_BUF_LENGTH;	// next alloc address 
   alloccnt -= HDLC_RX_BUF_LENGTH;	// remain bytes in alloc buffer 
  }

//
 bd--;					// Last BD 
 bd->Next = start;			// Assign last buffer descriptor
 start->Prev = bd; 			// Assign first buffer descriptor
} 

////////////////////////////////////////////////////////////////////////////////
// This function initiailze the HDLC Block /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HDLC_Init(void)
{
 U8  cn;				// channel num

// clear HDLC Statistics ///////////////////////////////////////////////////////
 HdlcClrErrReport();

 Disable_Int(nGLOBAL_INT);		// global disable int	
// Interrupt Vector Setup //////////////////////////////////////////////////////
 SysSetInterrupt(nHDLCTxA_INT, HDLCTxA_isr);	// HDLC A Tx ISR
 SysSetInterrupt(nHDLCRxA_INT, HDLCRxA_isr);	// HDLC A Rx ISR
 SysSetInterrupt(nHDLCTxB_INT, HDLCTxB_isr);	// HDLC B Tx ISR
 SysSetInterrupt(nHDLCRxB_INT, HDLCRxB_isr);	// HDLC B Rx ISR

// Enable Interrupts ///////////////////////////////////////////////////////////
 Enable_Int(nHDLCTxA_INT);		// enable HDLC A Tx Interrupt 
 Enable_Int(nHDLCRxA_INT);		// enable HDLC A Rx Interrupt 
 Enable_Int(nHDLCTxB_INT);		// enable HDLC B Tx Interrupt
 Enable_Int(nHDLCRxB_INT);		// enable HDLC B Rx Interrupt 

 for(cn = 0; cn <= 1; ++cn)
  {

//Reset All HDLC block /////////////////////////////////////////////////////////
   HCON(cn) |= 
    HCON_TxRS |				// Transmit Reset 	
    HCON_RxRS |				// Receive Reset 
    HCON_DTxRS|				// DMA Tx Reset
    HCON_DRxRS; 			// DMA Rx Reset

// Internal Register Setting for Tx/Rx operation ///////////////////////////////
   HMODE(cn) = gHMODE;			// HDLC mode register 
   HCON(cn)  = gHCON;			// HDLC control register 
   if(cn == 0) { HINTENA = gHINTENA; }  // HDLC int enable register
   else        { HINTENB = gHINTENB; } 

// Set Baud Rate Generator Counters ////////////////////////////////////////////
   HBRGTC(cn) = GetBaudRate(gHdlcBaudRate);	

// Set HDLC Channel Address Registers & Address Mask Register //////////////////
   HSAR0(cn) = 0x12345678;		// 
   HSAR1(cn) = 0xabcdef01;		// 
   HSAR2(cn) = 0xffffffff;		// 
   HSAR3(cn) = 0xaaaaaaaa;		// 
   HMASK(cn) = 0x00000000;		// 

   HMFLR(cn) = HDLC_MAX_FRAME_LENGTH;	// maximum frame length register 
   HRBSR(cn) = HDLC_RX_BUF_LENGTH;	// 
// Initialize Buffer Descriptor ////////////////////////////////////////////////
   TxBD_init(cn,0);			// Init Tx Buffer Descriptots	
   RxBD_init(cn);			// Init Rx Buffer Descriptots	

#ifdef USER_AREA
   HDLCRxUserRead [cn] = 
   HDLCRxUserWrite[cn] = (U32 *) (((U32) HDLCRxUserArea[cn]) | NonCache);
   HDLCRxUserFirst[cn] = (U32 *) (-1);
   HDLCRxUserCount[cn] = 0;
#endif 								// USER_AREA

   HCON(cn) |= (HCON_DRxEN|HCON_RxEN);	// HDLCA DMA Rx enable & Rx Enable 
  }


// Initialize Global Variables
 gHDLCRxDone = 0;			// no Rx interrupts
 Enable_Int(nGLOBAL_INT); 		// enable global interrupt
}


////////////////////////////////////////////////////////////////////////////////
// Send Data ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// cn   = channel number 
// data = data buffer 
// size = data length

int HDLCSend(int cn, U8 *data, int size)
{
 sHDLCBD * bd;				// current Tx BD 
 U32 * dst;				// destination buffer pointer 
 U32 * src;				// source pointer
 int len;				// length
 int cnt;				// BD count 

 if(cn < 0 || cn > 1 || data == 0 || size < 6)
  {					// invalid channel number, data buffer
   return(0);				// or data length
  }
 
 bd = (sHDLCBD *) gHDLCTxEnd[cn];	// End Tx BD pointer 
 len = size;				// sizeof sended data 
 while(len > 0)
  {
   if(bd == 0)
    {					// Null terminating List  
     return(0);				// not sufficient space to send data
    } 

   if(bd->BufferDataPtr & HDLCTxBD_DMA)
    {					// owner DMA
     return(0);				// not sufficient BDs to send data 
    }


   bd = bd->Next; 			// move to next BD 
   len -= HDLC_TX_BUF_LENGTH;		// remain data bytes
  }

// enough space in buffers /////////////////////////////////////////////////////

 cnt = 0;				
 bd = (sHDLCBD *) gHDLCTxEnd[cn];	// restore bd
 src = (U32 *) data;			// source address
 while(size > 0)
  {
   if(bd == 0)
    {					// null terminating list
     return(0);				// 
    } 

   if((bd->BufferDataPtr & HDLCTxBD_DMA))
    {					// 
     return(0);
    }

   if(size <= HDLC_TX_BUF_LENGTH)
    {
// Last Buffer Descriptor //////////////////////////////////////////////////////

     bd->Reserved = 
#ifdef LITTLE
      HDLCTxBD_Little  |		// Little-Endian 
#endif
      HDLCTxBD_WA0     |		// No invalid bytes
      HDLCTxBD_Last;			// Last buffer 
     bd->StatusLength = size;		// data length 	
    } 
   else 
    {
// not last BD /////////////////////////////////////////////////////////////////
     bd->Reserved = 
#ifdef LITTLE
      HDLCTxBD_Little  |		// Little-Endian 
#endif
      HDLCTxBD_WA0;			// No invalid bytes 

     bd->StatusLength = HDLC_TX_BUF_LENGTH;
    }

// copy sended data to destination buffer //////////////////////////////////////
   len = ((bd->StatusLength+3)>>2);	// length, words 
   dst = (U32 *) bd->BufferDataPtr;	// destination pointer
   while(len-- > 0)
    {					// word copy 
     *dst++ = *src++;
    } 

   cnt++; 				// used BD count 			
   size -= HDLC_TX_BUF_LENGTH;		// remain send data
   bd->BufferDataPtr |= HDLCTxBD_DMA;	// owner DMA 
   bd = bd->Next;			// move to next BD 	
  }

 Disable_Int(nGLOBAL_INT);
 gHDLCTxCount[cn] += cnt;		// Tx BD count
 Enable_Int(nGLOBAL_INT);

 gHDLCTxEnd[cn] = (U32) bd;		// reset end pointer 
// set Tx enable, DMA Tx enable bits ///////////////////////////////////////////
 HCON  (cn) |= HCON_TxEN | HCON_DTxEN;  // Tx enable & DMA Tx enable
 HINTEN(cn) |= HINTEN_DTxNOIE;		// enable not owner int 	
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// HDLC Channel A Tx ISR ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifdef HDLC_TX_ISR_DEBUG
#define HDLCTx_isr_debug(ch) put_byte(ch)
#else
#define HDLCTx_isr_debug(ch)
#endif					// HDLC_TX_ISR_DEBUG


void HDLCTx_isr(int cn)
{
 U32 stat;				// HDLC status register 
 U32 clear;				// HDLC status register clear bits

 stat  = HSTAT(cn);			// read status register 
 clear = 0;				// no cleared registers 

#ifdef HDLC_TX_ISR_DEBUG
 HDLCTxStatus[cn].TxISR++;
#endif 					// HDLC_TX_ISR_DEBUG

 HDLCTx_isr_debug('T');			// T = Tx ISR 
 if(stat & HSTAT_DTxFD)
  {
////////////////////////////////////////////////////////////////////////////////
// Tx Frame Done ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   sHDLCBD * bd;			// 
   U32 * data;				// 
   int i;

   HDLCTx_isr_debug('D');

// HDLC DMA Frame Done /////////////////////////////////////////////////////////
   HDLCTxStatus[cn].DMATxFD++;		// Tx Frame Done 
   clear |= HSTAT_DTxFD; 		// Clear DMA Tx Done Bit

// Check Tx Ownership in all Tx BD /////////////////////////////////////////////

   bd = (sHDLCBD *) gHDLCTxStart[cn];	// start Tx buffer descriptor 
   for(i = 0; i < HDLC_MAX_TX_DES; ++i) 
    { 
     data = & bd->BufferDataPtr;	// pointer to buffer data & owner bit 
     if((*data & HDLCTxBD_DMA))
      { 				// owner DMA not CPU
       break;				// break
      } 

     if((U32) bd == HDMATXPTR(cn))
      {
       break;
      } 

     if(bd->Next == 0)
      {
       Print("\nHDLCTxA_isr null next pointer");       
       break;				// circular buffers
      } 

     data = & bd->Reserved;		// pointer to reserved field

     if((*data & HDLCTxBD_Last))
      {
// last buffer in frame => check Tx Complete bit ///////////////////////////////
       data = & bd->StatusLength;	
       if((*data & HDLCTxBD_TxComp) == 0)
        { 
         HDLCTxStatus[cn].TxLBDNC++;	
        } 
      }  
     bd->Reserved     = 0;
     bd->StatusLength = 0;
     bd = bd->Next;			// move to next BD 
     gHDLCTxCount[cn]--;		// Tx Queue descriptors
     gHDLCTxStart[cn] = (U32) bd;	// reset current Tx BD
    }
  }

//////////////////////////////////////////////////////////////////////////////// 
// Check and clear other Tx bits /////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

 if(stat & HSTAT_TxFC)	
  {
// [4] Tx frame complete (TxFC) //////////////////////////////////////////////// 
   HDLCTxStatus[cn].TxFC++;		// Tx Complete
   clear |= HSTAT_TxFC;			// clear Tx Frame complete bit 
   HDLCTx_isr_debug('C');		// 
  }

 if(stat & HSTAT_TxFA)	
  {
// [5] Tx FIFO available  (TxFA) /////////////////////////////////////////////// 
   HDLCTxStatus[cn].TxFA++;		
/* during the DMA operation this bit allvais zero */
   HDLCTx_isr_debug('F');
  }

 if(stat & HSTAT_TxSCTS)
  {
// [7] Tx stored clear-to-send (TxSCTS) //////////////////////////////////////// 
   HDLCTxStatus[cn].TxSCTS++;
   clear |= HSTAT_TxSCTS; 		// clear stored clear-to-send
   HDLCTx_isr_debug('S');

//   if((stat & HSTAT_TxCTS))
//    {
//// nCTS low ////////////////////////////////////////////////////////////////////
//     if((HCON(cn) & HCON_DTxEN) == 0)
//      {
//       HDLCTx_isr_debug('E');
//       HCON(cn) |= HCON_DTxEN;
//      }
//    }  
//   else
//    {
//// nSTS high ///////////////////////////////////////////////////////////////////
//       HDLCTx_isr_debug('R');
//    } 
  }

 if(stat & HSTAT_TxU)
  {
// [8] Tx underrun (TxU) /////////////////////////////////////////////////////// 
   HDLCTxStatus[cn].TxU++;	        // Tx underrun statistics	
   clear |= HSTAT_TxU; 			// clear Tx Underrun bit 	
   HDLCTx_isr_debug('U');
  }

 if(stat & HSTAT_DTxABT)
  {
// DMA Tx abbort (due Tx underrun, CTS lost) ///////////////////////////////////  
   HDLCTxStatus[cn].DMATxABT++;		// DMA Tx Abort statistics
   clear |= HSTAT_DTxABT; 		// clear DMA Tx Abort bit 
// DMA Transmit enable bit DTxEN (in HCON) cleared  
   HDLCTx_isr_debug('A');
//   if((HCON(cn) & HCON_DTxEN) == 0)
//    {
//#ifdef HDLC_TX_ISR_DEBUG
//       HDLCTx_isr_debug('a');
//#endif	
//     HCON(cn) |= HCON_DTxEN;
//    }
  }

 if(stat & HSTAT_DTxNL)
  {
// [28] DMA Tx null list interrupt /////////////////////////////////////////////  
   HDLCTxStatus[cn].DMATxNL++;		// DMA Tx Null List statistics
   clear |= HSTAT_DTxNL; 		// clear DMA Tx Null List bit 
   HDLCTx_isr_debug('N');
  }

 if(stat & HSTAT_DTxNO)
  {
// [29] DMA Tx not owner (DTxNO) ///////////////////////////////////////////////  
   HDLCTxStatus[cn].DMATxNO++;		// DMA Tx not owner statistics 
   clear |= HSTAT_DTxNO; 		// clear DMA Tx Not Owner bit 
// DMA Transmit enable bit DTxEN (in HCON) cleared  
   HDLCTx_isr_debug('O');
   HINTEN(cn) &= ~HINTEN_DTxNOIE;
  }

// clear status bits ///////////////////////////////////////////////////////////   
 HSTAT(cn) = clear;			
}

////////////////////////////////////////////////////////////////////////////////
// HDLC Channel A Tx ISR ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HDLCTxA_isr(void)
{
 HDLCTx_isr(0);
}

////////////////////////////////////////////////////////////////////////////////
// HDLC Channel B Tx ISR ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HDLCTxB_isr(void)
{
 HDLCTx_isr(1);
}

////////////////////////////////////////////////////////////////////////////////
// clear Rx buffers ////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int HDLCRx_reset(int cn)
{
 sHDLCBD * bd;				// descriptor pointer 
 U32 intbit;				// HDLC channel Rx Int bit 
 int i;					// decriptor index

 if(cn < 0 || cn > 1)
  {					// invalid channel 
   return(0);				// number 
  } 

 intbit = (cn ? nHDLCRxB_INT : nHDLCRxA_INT);
 Disable_Int(intbit);			// disable HDLC Rx Int
 Clear_PendingBit(intbit);		// clear HDLC Rx pending bit

 HCON(cn) &= ~(HCON_RxEN | HCON_DRxEN);	// disable Rx & DMA Rx 
 HCON(cn) |=  (HCON_RxRS | HCON_DRxRS);	// reset Rx & DMA Rx

// 
#ifdef USER_AREA
 HDLCRxUserRead [cn] = 			// start position 
 HDLCRxUserWrite[cn] = 			// end position 
  (U32 *) ((U32) HDLCRxUserArea[cn] | NonCache);
 HDLCRxUserFirst[cn] = (U32 *) (-1);	// first buffer position 
 HDLCRxUserCount[cn] = 0;		// bytes count
#endif 								// USER_AREA

// 
#ifdef USER_AREA
 bd = (sHDLCBD *) gHDLCRxEnd[cn];	// moved by ISR 
#else 
 bd = (sHDLCBD *) gHDLCRxStart[cn];	// moved by Recv 
#endif 								// USER_AREA

 for(i = 0; i < HDLC_MAX_RX_DES; ++i) 
  {
   if((bd->BufferDataPtr & HDLCTxBD_DMA) != 0)
    {
// descriptor owner DMA (not CPU) => break /////////////////////////////////////
     break;
    }
   
   bd->BufferDataPtr |= HDLCTxBD_DMA;	// set DMA bit 
   bd->Reserved       = 0;		// clear reserved field
   bd->StatusLength   = 0;		// clear status length field

   if(bd->Next == 0)
    {					// null terminating list
     break;
    } 
   bd = bd->Next;			// move to next descriptor
  }

// 
 HDMARXPTR(cn)   = (U32) bd; 		// set current BD register
 gHDLCRxStart[cn]= (U32) bd;		// set start Rx BD 
 gHDLCRxEnd[cn]  = (U32) bd;		// set end Rx BD 
#ifndef USER_AREA
 gHDLCRxCount[cn]= 0;			// Rx BD count 
#endif 							// not def USER_AREA
 HSAR0(cn)  = 0x12345678;		// addr 0 reg
 HSAR1(cn)  = 0xabcdef01;		// addr 1 reg
 HSAR2(cn)  = 0xffffffff;		// addr 2 reg
 HSAR3(cn)  = 0xaaaaaaaa;		// addr 3 reg
 HMASK(cn)  = 0x00000000;		// addr mask reg
 HMFLR(cn)  = HDLC_MAX_FRAME_LENGTH;	// maximum frame length register 
 HRBSR(cn)  = HDLC_RX_BUF_LENGTH;	// recv buffer size 
 HCON(cn)   = gHCON | HCON_DRxEN | HCON_RxEN;	// restore control register 	
 HINTEN(cn) = (cn ? gHINTENB : gHINTENA); 	// restore int reg
 Enable_Int(intbit);			// enable rx int
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// return count not transmitted Tx BD //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int HDLCTxBD_count(int cn)
{
 if(cn < 0 || cn > 1)
  {					// invalid channel 
   return(0);				// number 
  }
 return(gHDLCTxCount[cn]);
}


////////////////////////////////////////////////////////////////////////////////
// clear Tx buffer descriptosr /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int HDLCTx_reset(int cn)
{
 sHDLCBD * bd;				// descriptor pointer 
 U32 intbit;
 int i;					// descriptor index

 if(cn < 0 || cn > 1)
  {					// invalid channel number 
   return(0);				// 
  }

 intbit = (cn ? nHDLCTxB_INT : nHDLCTxA_INT);
 Disable_Int(intbit);
 Clear_PendingBit(intbit);

 bd = (sHDLCBD *) HDMATXPTR(cn);		// current Tx bd

 HCON(cn) &= ~(HCON_TxEN | HCON_DTxEN);	// disable Tx & DMA Tx
 HCON(cn) |=  (HCON_TxRS | HCON_DTxRS);	// reset Tx & DMA Tx 

// Check Tx Ownership in all Tx BD /////////////////////////////////////////////
 for(i = 0; i < HDLC_MAX_TX_DES; ++i) 
  { 
   if((bd->BufferDataPtr & HDLCTxBD_DMA) == 0) 		
    { 				
// owner CPU => break ////////////////////////////////////////////////////////// 
     break;			
    } 

   bd->BufferDataPtr &= ~HDLCTxBD_DMA;	// Owner CPU
   bd->Reserved     = 0;		// clear reserved field
   bd->StatusLength = 0;		// clear status length field

   if(bd->Next == 0)
    {
     Print("\nHDLC Tx BD null next pointer");       
     break;				// circular buffers
    } 
   bd = bd->Next;			// move to next BD 
  }

 gHDLCTxStart[cn] = (U32) bd; 		// set start Tx BD
 gHDLCTxEnd[cn]   = (U32) bd; 		// set end   Tx BD
 gHDLCTxCount[cn] = 0;			// Tx BD count 
 HDMATXPTR(cn)    = (U32) bd;		// set current Tx BD pointer 
 HCON(cn)         = gHCON;		// restore HCON
 HINTEN(cn) = (cn ? gHINTENB:gHINTENA); // restore HINTEN
 Enable_Int(intbit);
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// save Rx buffer //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
#ifdef USER_AREA
static U32 GetRxBDData(int cn, U32 buf, U32 status, int len)
{
 U32 * data;				// 
 U32 * ptr;				// write pointer 	
 U32 * start; 				// queue start pointer  
 U32 * end; 				// queue end pointer 
 int dcnt = 0;				// HDLCRxUserCount increment 
 int wlen;
  
 wlen = (len + 3) / 4;
 if(wlen + 1 + HDLCRxUserCount[cn] > HDLC_USER_AREA_SIZE )
  {
   Print("\nGetRxBDData buffer full %d %d", len, HDLCRxUserCount[cn]);
   return(0);
  } 

// 
// start = (U32 *) HDLCRxUserArea [cn]  /*| NonCache*/;	// start pointer 
// end   = start + HDLC_USER_AREA_SIZE; 			// end pointer 
// ptr   = (U32 *) HDLCRxUserWrite[cn]  /*| NonCache*/;	// write pointer 

 start = (U32 *) ((U32)HDLCRxUserArea [cn] | NonCache);
 ptr   = (U32 *) ((U32)HDLCRxUserWrite[cn] | NonCache);
 end   = start + HDLC_USER_AREA_SIZE; 			
 if(ptr >= end) { ptr = start; }

//
 if(status & HDLCRxBD_F) 		// [22] First In Frame (F)
  {
   U32 * fptr;
////////////////////////////////////////////////////////////////////////////////
// first buffer in frame => write to queue preample, status, length ////////////
////////////////////////////////////////////////////////////////////////////////

   fptr = (U32 *) ((U32) HDLCRxUserFirst[cn] | NonCache);
   if(fptr != (U32 *) -1)
    {
     if(fptr >= end) { fptr = start; }// rotate to queue start
     *fptr |= ((U32)HDLCRxBD_L << 16);
    }  

   if(ptr >= end) { ptr = start; }	// rotate to begin queue
   if((status & HDLCRxBD_L) == 0) 	
    {					
// last bit not set => save first pointer //////////////////////////////////////
     HDLCRxUserFirst[cn] = ptr;		// set first buffer pointer  
    }
   else
    {
// first and last frame buffer ///////////////////////////////////////////////// 
     HDLCRxUserFirst[cn] = (U32 *) -1;	// clear first pointer 
    }
     
   *ptr = ((status & 0xFFFF) << 16) | (len & 0xFFFF);
   if(++ptr >= end) { ptr = start; }	// rotate to queue start
   dcnt += 1;				// writed words count
  }
 else
  {
////////////////////////////////////////////////////////////////////////////////
// not first buffer in frame ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
   U32 * fptr;     			// pointer to first buffer
   int flen;      			
// 
   fptr = (U32 *) ((U32) HDLCRxUserFirst[cn] | NonCache);
   if(fptr == ((U32 *) -1))
    {
     Print("\nHDLC Rx Internal error");
     return(0);
    }

   if(fptr >= end) { fptr = start; }	//    rotate to queue begin 
   *fptr |= ((status & 0xFFFF) << 16);

    flen = *fptr & 0xFFFF;
   *fptr = (*fptr & 0xFFFF0000) | (len & 0xFFFF);
    len -= flen;

   if((status & HDLCRxBD_L) != 0)
    {
// last buffer => clear pointer to first /////////////////////////////////////// 
     HDLCRxUserFirst[cn] = (U32 *) -1;	// clear first pointer 
    }
  }

// source address //////////////////////////////////////////////////////////////

 data = ((U32 *) buf);
 wlen = (len + 3) / 4;
 if(ptr >= end) { ptr = start; }	// rotate to queue start
 while(wlen-- > 0)
  {
   *ptr++ = *data++;			// 
   if(ptr >= end) { ptr = start; }	// rotate to queue start
   dcnt += 1;				// bytes is writen
  }

 HDLCRxUserCount[cn]+= dcnt;		// bytes in queue
 HDLCRxUserWrite[cn] = ptr;		// reset write position 
 return(1);
}
#endif 								// USER_AREA


#ifdef USER_AREA
int HDLCRecv(int cn, U32 * status, U8 * buf, int * size)
{
 U32 intbit;
 U32 * data;
 U32 * start; 				// queue start address
 U32 * end; 				// queue end address 
 U32 * ptr;				// read pointer 
 int len;				// data length, bytes 
 int wlen;				// 
 int wsize;				// 
 int dcnt = 0;

 Disable_Int(nGLOBAL_INT);
 if(HDLCRxUserCount[cn] < 1)
  {
   Enable_Int(nGLOBAL_INT);
   return(0);
  } 

 start = (U32 *) ((U32) HDLCRxUserArea[cn] | NonCache);	// queue start pointer 
 ptr   = (U32 *) ((U32) HDLCRxUserRead[cn] | NonCache);	// queue read pointer 
 end   = start + HDLC_USER_AREA_SIZE; 			// queue end pointer 
 if(ptr >= end) { ptr = start; }

 *status = (*ptr >> 16) & 0xFFFF;	// read status 
 if((*status & HDLCRxBD_L) == 0)
  {
// last bit not set => wait Rx last buffer /////////////////////////////////////
   Enable_Int(nGLOBAL_INT);
   return(0);
  }

 len = *ptr & 0xFFFF;			// read length 
 if(++ptr >= end) { ptr = start; }	// rotate read pointer to queue start

// read words from queue ///////////////////////////////////////////////////////
 dcnt = 1;				// readed header
 data = (U32 *) buf;			
 
 wlen  = len >> 2;			// frame length, full words
 wsize = *size >> 2;			// buffer size, full words
 while(wlen-- > 0)
  {
   if(wsize-- > 0)
    {
     *data++ = *ptr;			// 
    }
// 
   dcnt++;				// readed words from queue
   ptr++;				// move pointer 
   if(ptr >= end) { ptr = start; }	// rotate read pointer to queue start
  } 

 if((len & 0x3) != 0) 
  {
   wlen = len & 0xFFFFFFFC;  		// dst pos 
   wsize = *ptr;			// read word from queue 
   if(++ptr >= end) { ptr = start; }	// rotate read pointer to start
   dcnt++;				// readed words  

// 
   while(wlen < len)
    {
     if(wlen >= *size)
      {
       break;
      }

     *(buf + wlen++) = (U8) wsize & 0xFF;
      wsize >>= 8;
    }
  }
 
*size = len;				// frame length
 HDLCRxUserCount[cn]-= dcnt;		// words in queue
 HDLCRxUserRead[cn]  = ptr;		// reset read pointer 
 Enable_Int(nGLOBAL_INT);	
 return(1);
}
#else
int HDLCRecv(int cn, U32 * status, U8 * buf, int * size)
{
 sHDLCBD * bd;				// current Rx BD 
 U32 stat;				// BD status 
 int cnt;				// BD count 
 int num;				// BD number
 int flen;				// total length 
 int len;				// length 
 U32 * dst;				// destination address 
 U32 * src;				// source address 
 int wlen;				// length, words 
 int pos;				// destination position


 if(cn < 0 || cn > 1 || status == 0 || buf == 0 || size == 0)
  {					// invalid channel number, data buffer
   return(-1);				// or data length ...
  }

 cnt = gHDLCRxCount[cn];		// Rx BD count 
 if(cnt == 0)
  {					// no Rx data
   return(0);
  } 
 else
 if(cnt < 0)
  {
   Print("\nHDLCRecv internal error. Rx BD count < 0");
   return(-1);
  } 

 
 bd = (sHDLCBD *) gHDLCRxStart[cn];
 for(num = 0; num < cnt; ++num)
  {
   if(bd == 0)
    {					// Null terminating List  
     Print("\nHDLCRecv error, BD == 0");
     return(-1);	
    } 

   if((bd->BufferDataPtr & HDLCRxBD_DMA))
    {					// owner DMA
     return(0);				//
    }

// read status & check last BD in frame bit
   stat = (bd->StatusLength >> 16) & 0xffff;	
   if((stat & HDLCRxBD_L) != 0 || (num > 0 && (stat & HDLCRxBD_F)))
    {					
// last BD in frame, or ommit last BD
     break;				// 
    }  
   bd = bd->Next;
  }

 if(num >= cnt)
  {					// no last BD in frame 
   return(0); 
  }

// 
 bd = (sHDLCBD *) gHDLCRxStart[cn];	// start buffer descriptor 
*status = 0;				// frame status 
 flen = 0;				// frame length 
 pos = 0;				// destination buffer position 
 for(num = 0; num < cnt; ++num)
  {
   int tmp;

   if(bd->BufferDataPtr & HDLCRxBD_DMA)
    {					// owner DMA
     Print("\nHDLCRecv internal error. Unexpected DMA bit.");
     return(0);
    }

// read BD status, data length
   stat = (bd->StatusLength >> 16) &0xffff;	
   len  = (bd->StatusLength & 0xFFFF);		

   if(num > 0 && (stat & HDLCRxBD_F))
    {					// find first BD, ommit last BD
     break;
    }  

   *status |= stat;			// frame status

// 
   tmp = flen;
   flen = len;   			// total length 
   len -= tmp;
   if(len < 0)
    {		
     Print("\nHDLCRecv internal error. Length < 0");
     return(-1);	
    } 

//
   wlen = (len >> 2);
   dst = (U32 *)(buf + pos);
   src = (U32 *) bd->BufferDataPtr;
   while(wlen-- > 0)
    {
     pos += 4; 
     if(pos <= *size)
      {
       *dst++ = *src++;
      }
    }

   if((len & 0x3) != 0) 
    {
     U8 * bsrc;
     wlen = len & 0xFFFFFFFC; 
     bsrc = (U8 *) bd->BufferDataPtr + wlen;
     while(wlen++ < len)
      {
       if(pos < *size)
        {
        *(buf + pos) =  *bsrc++;
        }
       pos++;
      }
    }

   bd->Reserved       = 0;		// clear BD reserved field 
   bd->StatusLength   = 0;		// clear status & length field
   bd->BufferDataPtr |= HDLCRxBD_DMA;	// owner DMA
   bd = bd->Next;			// move to next descriptor 

   if((stat & HDLCRxBD_L))
    {					// last BD in frame
     break;
    }  
  }

// save frame length
 *size = flen;

 Disable_Int(nGLOBAL_INT);		// disable global int 
 gHDLCRxCount[cn]-= (num + 1);		// decrement Rx BD count 	
 gHDLCRxStart[cn] = (U32) bd;		// move start BD 
// enable DMA Rx, may be cleared if ovner of current Rx decriptor is CPU 
 HCON(cn)  |= HCON_DRxEN;   		// enable DMA Rx 
 Enable_Int(nGLOBAL_INT);		// enable global int
 return(1);
}
#endif 						// USER_AREA


#ifdef HDLC_RX_ISR_DEBUG
#define HDLCRx_isr_debug(ch) put_byte(ch)
#else
#define HDLCRx_isr_debug(ch)
#endif					// HDLC_RX_ISR_DEBUG

void HDLCRx_isr(int cn)
{
 U32 stat;				// HDLC channel status
 U32 clear;				// cleared bits

 stat = HSTAT(cn);			// read HDLC status register 
 clear = 0;				// start cleared bits	

#ifdef HDLC_RX_ISR_DEBUG
 HDLCRxStatus[cn].RxISR++;
#endif 					// HDLC_RX_ISR_DEBUG

 HDLCRx_isr_debug('R'); 
 if(stat & HSTAT_DRxFD)
  {
////////////////////////////////////////////////////////////////////////////////
// [24] DMA Rx frame done every received frame (DRxFD) /////////////////////////
////////////////////////////////////////////////////////////////////////////////

   sHDLCBD * bd;
   U32 status;
   U32 len;
   int i;

   HDLCRx_isr_debug('D'); 

   clear |= HSTAT_DRxFD;		// clear DMA Rx frame done bit 
   HDLCRxStatus[cn].DRxFD++;		// DMA Rx frame done statistics
	
   bd = (sHDLCBD *) gHDLCRxEnd[cn];	// end buffer descriptor 
   for(i = 0; i < HDLC_MAX_RX_DES; ++i) 
    {
     if((bd->BufferDataPtr & HDLCRxBD_DMA) != 0)
      {
// descriptor owner DMA (not CPU) => break /////////////////////////////////////
       break;
      }

     if((U32)bd == HDMARXPTR(cn))
      {					
       break;
      } 

     status = (bd->StatusLength >> 16) &0xffff;	// read status 
     len    =  bd->StatusLength & 0xffff;	// read length 		

     if(
       status & 
        (
         HDLCRxBD_CD  |			// CD lost 
         HDLCRxBD_CRC |			// CRC error 
         HDLCRxBD_NO  |			// Non octet alignment 
         HDLCRxBD_OV  |			// Overun 
         HDLCRxBD_DTM |			// DPLL two miss occurs
         HDLCRxBD_ABT |			// received frame abborted
         HDLCRxBD_FLV 			// frame length violation 
        )
      )
      {
// Rx frame error //////////////////////////////////////////////////////////////
       if((status & HDLCRxBD_CRC) != 0)
        {
         HDLCRxStatus[cn].RxCRCE++;	// Rx CRC Error	
        }

       if((status & HDLCRxBD_NO) != 0)
        {
         HDLCRxStatus[cn].RxNO++;	// Rx Non Octet
        }

       if((status & HDLCRxBD_OV) != 0)
        {
         HDLCRxStatus[cn].RxOV++;	// Rx Overrun
        } 

       if((status & HDLCRxBD_ABT) != 0)
        {
         HDLCRxStatus[cn].RxABT++;	// Rx Abort
        }

       if((status & HDLCRxBD_FLV) != 0)
        {
         HDLCRxStatus[cn].RxFV++;	// Rx Frame Valid
        }

       if((status & HDLCRxBD_DTM) != 0)
        {
         HDLCDPLLStatus[cn].DPLLTM++;
        } 
      }  

// save Rx buffer (not frame) ////////////////////////////////////////////////// 

#ifdef USER_AREA
     GetRxBDData(cn, bd->BufferDataPtr, status, len); 
// clear BD fields and move to next BD /////////////////////////////////////////
     bd->Reserved       = 0;		// clear BD reserved field 
     bd->StatusLength   = 0;		// clear status & length field
     bd->BufferDataPtr |= HDLCRxBD_DMA;	// owner DMA
#else
     gHDLCRxCount[cn]++;
#endif 								// USER_AREA
     bd = bd->Next;			// to next BD 
    }

   gHDLCRxEnd[cn] = (U32) bd;		// set end Rx BD 
   gHDLCRxDone = (0x1 << cn);		// set Rx done channel bit 
  }

// [9] Rx FIFO available (RxFA) ////////////////////////////////////////////////
 if(stat & HSTAT_RxFA)
  {
   HDLCRxStatus[cn].RxFA++;		// Rx FIFO Avail
   clear |= HSTAT_RxFA;			// clear Rx FIFO Available bit
   HDLCRx_isr_debug('A');
  }

// [11] Rx flag detected (RxFD) ////////////////////////////////////////////////
 if(stat & HSTAT_RxFD)
  {
   U32 hinten;
   HDLCRxStatus[cn].RxFD++;		// Rx Flag Detect statistics
   clear |= HSTAT_RxFD;			// clear Rx Flag detect bit
   hinten = HINTEN(cn); 
   if(hinten & HINTEN_RxFDIE)
    {
/* clear flag detect int enable set idle detect int enable */
     HINTEN(cn) = ((hinten & ~HINTEN_RxFDIE) | HINTEN_RxIDLEIE);
    }
   HDLCRx_isr_debug('F');
  }

// [13] Rx stored data carrier detected (RxSDCD) ///////////////////////////////
 if(stat & HSTAT_RxSDCD)
  {
   HDLCRxStatus[cn].RxSDCD++;		// DCD store statistics
   clear |= HSTAT_RxSDCD;		// clear DCD store bit
   HDLCRx_isr_debug('C');

   if((HCON(cn) & HCON_DRxEN) == 0) 
    {				
     HCON(cn) |= HCON_DRxEN;
    }

   if((HCON(cn) & HCON_RxEN) == 0) 
    {				
     HCON(cn) |= HCON_RxEN;
    }
  }

// [14] Rx frame valid (RxFV) //////////////////////////////////////////////////

 if(stat & HSTAT_RxFV)
  {
   HDLCRxStatus[cn].RxFV++;		// Rx Frame Valid statistics
   clear |= HSTAT_RxFV;			// [14] Rx frame valid (RxFV)
   HDLCRx_isr_debug('V');
  }

// [15] Rx idle (RxIDLE) ///////////////////////////////////////////////////////

 if(stat & HSTAT_RxIDLE)
  {
   U32 hinten;
   HDLCRxStatus[cn].RxIDLE++;		// Rx Idle statistics
   clear |= HSTAT_RxIDLE;		// clear Rx IDLE bit 
   HDLCRx_isr_debug('I');
/* clear idle detect int enable set flag detect int enable */
   hinten = HINTEN(cn);
   if(hinten & HINTEN_RxIDLEIE)
    {
     HINTEN(cn) = ((hinten & ~HINTEN_RxIDLEIE) | HINTEN_RxFDIE);
    }
  }

// [16] Rx abort (RxABT) ///////////////////////////////////////////////////////

 if(stat & HSTAT_RxABT)
  {
   HDLCRxStatus[cn].RxABT++;		// Rx abbort statistics
   clear |= HSTAT_RxABT;		// clear Rx abort bit 
   HDLCRx_isr_debug('B');
  }

// [17] CRC error frame interrupt enable ///////////////////////////////////////

 if(stat & HSTAT_RxCRCE) 
  {
   HDLCRxStatus[cn].RxCRCE++;		// Rx CRC statistics
   clear |= HSTAT_RxCRCE;		// clear CRC error bit
   HDLCRx_isr_debug('C');
  }

// [18] Rx non-octet align (RxNO) //////////////////////////////////////////////

 if(stat & HSTAT_RxNO)  
  {
   HDLCRxStatus[cn].RxNO++;		// non-octet align statistics 
   clear |= HSTAT_RxNO;			// clear bit 
   HDLCRx_isr_debug('L');
  }

// [19] Rx overrun (RxOV) //////////////////////////////////////////////////////
 if(stat & HSTAT_RxOV)
  {
   HDLCRxStatus[cn].RxOV++;		// Rx Overrun statistics
   clear |= HSTAT_RxOV;			// clear Rx Overrun bit   
   HDLCRx_isr_debug('o');
  }

 if(stat & HSTAT_RxMOV) 
  {
   HDLCRxStatus[cn].RxMOV++; 		// memory overflow statistics
   clear |= HSTAT_RxMOV;		// clear Rx memory overflow bit
// If this bit is set, DRxEN bit is cleared (RX DMA disabled).
   HDLCRx_isr_debug('M');
  }	

// [23] Rx internal error (RxIERR) /////////////////////////////////////////////

 if(stat & HSTAT_RxIERR)
  {
   HDLCRxStatus[cn].RxIERR++;		// Rx Inter Error statistics
   clear |= HSTAT_RxIERR;		// clear Rx internal bit  
   HDLCRx_isr_debug('E');
  }

// [25] DMA Rx null list (DRxNL) ///////////////////////////////////////////////
 if(stat & HSTAT_DRxNL) 
  {
   HDLCRxStatus[cn].DRxNL++;		// DMA Rx null list statistics
   clear |= HSTAT_DRxNL;		// clear Rx null list bit 
// If this bit is set, DRxEN bit is cleared (RX DMA disabled)
   HDLCRx_isr_debug('L');
  }	

// [26] DMA Rx not owner (DRxNO) ///////////////////////////////////////////////
 if(stat & HSTAT_DRxNO)
  {
   HDLCRxStatus[cn].DRxNO++;		// Rx DMA NO
   clear |= HSTAT_DRxNO;		// clear Rx Not owner bit  
// If this bit is set, DRxEN bit is cleared (RX DMA disabled)
   if((HCON(cn) & HCON_DRxEN) == 0)
    {
     HCON(cn) |= HCON_DRxEN;
    }
   HDLCRx_isr_debug('O');
  }

// [30] DPLL one clock missing (DPLLOM) ////////////////////////////////////////
 if(stat & HSTAT_DPLLOM)
  {
   HDLCDPLLStatus[cn].DPLLOM++;		// DPLL Statistics 
   clear |= HSTAT_DPLLOM;		// 
   HDLCRx_isr_debug('1');
  }

 if(stat & HSTAT_DPLLTM)
  {
   HDLCDPLLStatus[cn].DPLLTM++;		// DPLL Statistics 
   clear |= HSTAT_DPLLOM;		// 
   HDLCRx_isr_debug('2');
  }

 HSTAT(cn) = clear;
}

void HDLCRxA_isr(void)
{
 HDLCRx_isr(0);
}

void HDLCRxB_isr(void)
{
 HDLCRx_isr(1);
}

