/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/* snds.a                                 SNDS100 Board version 1.0      */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*     SNDS100 for KS32C5000, KS32C50100 ASSEBLER SYSTEM HEADER FILE     */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*      Young Sun KIM   09-25-1998      Created initial version 1.0      */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002,2003,2004	*/
/*/			*/

/************************************************************************/
/* Format of the Program Status Register                                */
/************************************************************************/
/*                                                                      */
/* 31  30  29   28         7   6   5   4   3   2   1   0                */
/*+---+---+---+---+--ss--+---+---+---+---+---+---+---+---+              */
/*| N | Z | C | V |      | I | F | T |     M4 ~ M0       |              */
/*+---+---+---+---+--ss--+---+---+---+---+---+---+---+---+              */
/* 									*/	 
/* 7:0 	(C)onftrol bits						  	*/ 
/* 15:8   	e(X)tension bits					*/
/* 23:16	(S)tatus bits						*/
/* 31:24	(F)lag bits						*/  
/*                                                                      */
/************************************************************************/

/************************************************************************/
/* Processor Mode and Mask                                              */
/************************************************************************/
#define FBit	     	         0x40   /* FIQ &=0x */
#define IBit	                 0x80   /* IRQ */
#define LOCK_MASK                0xC0   /* Interrupt lockout mask value */
#define MODE_MASK                0x1F   /* Processor Mode Mask */

#define UDF_MODE                 0x1B   /* Undefine Mode(UDF) */
#define ABT_MODE                 0x17   /* Abort Mode(ABT) */
#define SUP_MODE                 0x13   /* Supervisor Mode (SVC) */
#define IRQ_MODE                 0x12   /* Interrupt Mode (IRQ) */
#define FIQ_MODE                 0x11   /* Fast Interrupt Mode (FIQ) */
#define USR_MODE                 0x10   /* User Mode(USR) */

/*************************************************************************/
/* SYSTEM STACK MEMORY  : 8K bytes system stacks are defined at memory.a */
/*************************************************************************/
#ifdef LED_ONLY
#define DRAM_BASE           0x3FE0000   /* Internal SRAM */
#define DRAM_LIMIT          0x3FE2000   /* */
#define USR_STACK_SIZE            256   /* User stack size */ 
#define UDF_STACK_SIZE            128   /* Undefined mode stack size */
#define ABT_STACK_SIZE            128   /* Abort mode stack size */
#define IRQ_STACK_SIZE            128   /* IRQ mode stack size */
#define FIQ_STACK_SIZE            128   /* FIQ mode stack size */
#define SUP_STACK_SIZE            128   /* Supervisor mode stack size */
#else 
#define DRAM_BASE           0x1000000   /* Address of DRAM start byte */
#define DRAM_LIMIT          0x2000000   /* 16 Mbytes, next byte after DRAM */
#define USR_STACK_SIZE           2048   /* User stack size */ 
#define UDF_STACK_SIZE            512   /* Undefined mode stack size */
#define ABT_STACK_SIZE            512   /* Abort mode stack size */
#define IRQ_STACK_SIZE           2048   /* IRQ mode stack size */
#define FIQ_STACK_SIZE           2048   /* FIQ mode stack size */
#define SUP_STACK_SIZE           2048   /* Supervisor mode stack size */
#endif 							/* LED_ONLY */
    
/*************************************************************************/
/*  SYSTEM CLOCK                                                         */
/*************************************************************************/
#ifdef KS32C50100
#define fMCLK      	           50   /* System Clock MHz */
#else 
#define fMCLK     	           50   /* System Clock MHz */
#endif							/* KS32C50100 */
									
/************************************************************************/
/*  SYSTEM MEMORY CONTROL REGISTER EQU TABLES                           */
/************************************************************************/
 
/************************************************************************/
/* -> EXTDBWTH:Memory Bus Width register 				*/
/*		Offset = 0x3010, Reset Value = 0x0000000		*/
/* 									*/
/* [ 1: 0] DSR0 - Data bus width for ROM/SRAM/FLASH bank 0 		*/
/*									*/
/* DSR0 is read-only data at the B0SIZE[1:0] pins. DSR0 is read-only    */
/* because ROM/SRAM/FLASH bank 0 is used to boot the ROM while the	*/
/* data bus width for ROM/SRAM/FLASH bank 0 is set using B0SIZE[1:0]	*/
/* 									*/
/* [ 3: 2] DSR1 - Data bus width for ROM/SRAM/FLASH bank 1 		*/
/* [ 5: 4] DSR2 - Data bus width for ROM/SRAM/FLASH bank 2 		*/
/* [ 7: 6] DSR3 - Data bus width for ROM/SRAM/FLASH bank 3 		*/
/* [ 9: 8] DSR4 - Data bus width for ROM/SRAM/FLASH bank 4 		*/
/* [11:10] DSR5 - Data bus width for ROM/SRAM/FLASH bank 5 		*/
/*		   00 = Disable 					*/
/*		   01 = Byte (8 bits) 					*/
/*		   10 = Half-word (16 bits) 				*/
/*		   11 = Word (32 bits) 					*/
/*									*/
/* [13:12] DSD0 - Data bus width for DRAM bank 0 			*/
/* [15:14] DSD1 - Data bus width for DRAM bank 1 			*/
/* [17:16] DSD2 - Data bus width for DRAM bank 2 			*/
/* [19:18] DSD0 - Data bus width for DRAM bank 3 			*/
/*		   00 = Disable 					*/
/*		   01 = Byte (8 bits) 					*/
/*		   10 = Half-word (16 bits) 				*/
/*		   11 = Word (32 bits) 					*/
/*           								*/
/* [21:20] DSX0 - Data bus width for external I/O bank 0 		*/
/* [23:22] DSX1 - Data bus width for external I/O bank 1 		*/
/* [25:24] DSX2 - Data bus width for external I/O bank 2 		*/
/* [27:26] DSX0 - Data bus width for external I/O bank 3 		*/
/*		   00 = Disable 					*/
/*		   01 = Byte (8 bits) 					*/
/*		   10 = Half-word (16 bits) 				*/
/*		   11 = Word (32 bits) 					*/
/************************************************************************/
/* ROM/SRAM/FLASH */
#define DSR0  	               (1<<0)   /* ROM0 0 : Disable */
					/*      1 : Byte */
					/*      2 : Half-Word */
					/*      3 : Word */
#define DSR1  	              (1<< 2)   /* ROM1   : (Byte) */
#define DSR2  	              (1<< 4)   /* ROM2   : (Byte) */ 	
#define DSR3  	              (0<< 6)   /* ROM3   : (disable) */
#define DSR4  	              (0<< 8)   /* ROM4   : (disable) */
#define DSR5  	              (0<<10)   /* ROM5   : (disable) */
/* Data bus width for DRAM bank 0,1,2,3 */
#define DSD0  	              (3<<12)   /* DRAM0  : (Word) */  
#define DSD1  	              (0<<14)   /* DRAM1  : (disable) */
#define DSD2  	              (0<<16)   /* DRAM2  : (disable) */
#define DSD3  	              (0<<18)   /* DRAM3  : (disable) */
/* Data bus width for external I/O banks 0,1,2,3 */
#define DSX0  	              (0<<20)   /* EXTIO0 : (disable) */
#define DSX1  	              (0<<22)   /* EXTIO1 : (disable) */
#define DSX2  	              (0<<24)   /* EXTIO2 : (disable) */
#define DSX3  	              (0<<26)   /* EXTIO3 : (disable) */	

#define rEXTDBWTH   (DSR0+DSR1+DSR2+DSR3+DSR4+DSR5+DSD0+DSD1+DSD2+DSD3+DSX0+DSX1+DSX2+DSX3)
									
/************************************************************************/
/* -> ROMCON0 :ROM Bank0 Control register 				*/
/* 		Offset Address=0x3014, Reset Value=0x20000060		*/
/*									*/
/* [1:0] 	PCM Page mode configuration  				*/
/*		00 = Normal Rom 					*/
/*		01 = 4-word page 					*/
/*		10 = 8-word page 					*/
/*		11 = 16-word page					*/
/* [3:2]	tPA Page address access time 				*/
/*		00 = 5 cycles		01 = 2 cycles			*/
/*		10 = 3 cycles		11 = 4 cycles			*/
/*									*/
/* [6:4]	tACC Programmable access cycle 				*/
/* 		000= Disable bank 	001= 2 cycles			*/
/* 		010= 3 cycles		011= 4 cycles			*/
/* 		100= 5 cycles		101= 6 cycles			*/
/* 		110= 7 cycles		111= Reserved			*/
/* 									*/
/* [19:10]	ROM/SRAM/Flash bank base pointer 			*/
/* [29:20]	ROM/SRAM/Flash bank next pointer 			*/
/* 									*/
/************************************************************************/

#define initROMBasePtr0   (0x000<<10)   /* =0x0000000 ;  0 */
#define initROMEndPtr0    (0x020<<20)   /* =0x0200000 ;  2 */
#define workROMBasePtr0   (0x200<<10)   /* =0x2000000 ; 32 */
#define workROMEndPtr0    (0x220<<20)   /* =0x2200000 ; 34 */

/* Page Mode Configuration (PCM)*/
#define PMC0                      0x0   /* 0x0=Normal ROM, 0x1=4Word Page */ 
                            	        /* 0x2=8Word Page, 0x3=16Word Page */
/* Page address access time */ 
#define rTpa0                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle ; 5 Cycles */
                                        /* 0x2=3Cycle, 0x3=4Cycle */ 
/* Programmable access cycle */
#define rTacc0               (0x4<<4)   /* 0x0=Disable, 0x1=2Cycle	; */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
                                        /* 0x4=5Cycle, 0x5=6Cycle */
                            	        /* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON0     (initROMEndPtr0+initROMBasePtr0+rTacc0+rTpa0+PMC0)
#define workROMCON0     (workROMEndPtr0+workROMBasePtr0+rTacc0+rTpa0+PMC0)

/************************************************************************/
/* -> ROMCON1 :ROM Bank1 Control register 				*/
/*		Offset Address = 0x3018	Reset Value    = 0x00000060	*/
/************************************************************************/ 
#define initROMBasePtr1   (0x020<<10) /*=0x0200000 ;  2 */
#define initROMEndPtr1    (0x040<<20) /*=0x0400000 ;  4 */
#define workROMBasePtr1   (0x220<<10) /*=0x2200000 ; 34 */
#define workROMEndPtr1    (0x240<<20) /*=0x2400000 ; 36 */

/* Page Mode Configuration (PCM) */
#define PMC1                      0x0   /* 0x0=Normal ROM, 0x1=4Word Page ; Normal */
                    	  		/* 0x2=8Word Page, 0x3=16Word Page */
/* Page address access time */
#define rTpa1                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle	; 5 Cycles  */
                                        /* 0x2=3Cycle, 0x3=4Cycle  */
/* Programmable access cycle */
#define rTacc1               (0x6<<4)   /* 0x0=Disable, 0x1=2Cycle ; 7 Cycles */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
                                        /* 0x4=5Cycle, 0x5=6Cycle */
                                        /* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON1     (initROMEndPtr1+initROMBasePtr1+rTacc1+rTpa1+PMC1)
#define workROMCON1     (workROMEndPtr1+workROMBasePtr1+rTacc1+rTpa1+PMC1)

/************************************************************************/
/* -> ROMCON2 :ROM Bank2 Control register 				*/
/*		Offset Address = 0x301C Reset Value    = 0x00000060) 	*/
/************************************************************************/
#define initROMBasePtr2   (0x040<<10)	/*=0x0400000 ;  4 */ 
#define initROMEndPtr2    (0x060<<20)	/*=0x0600000 ;  6 */
#define workROMBasePtr2   (0x240<<10)	/*=0x2400000 ; 36 */ 
#define workROMEndPtr2    (0x260<<20)	/*=0x2600000 ; 38 */

/* Page Mode Configuration */
#define PMC2                      0x0   /* 0x0=Normal ROM, 0x1=4Word Page; Normal */
                        	        /* 0x2=8Word Page, 0x3=16Word Page */
/* Page Access Time */
#define rTpa2                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle	; 5 Cycles */
                              		/* 0x2=3Cycle, 0x3=4Cycle */
/* Programmable access cycle */
#define rTacc2             (0x6 << 4)   /* 0x0=Disable, 0x1=2Cycle */
                                	/* 0x2=3Cycle, 0x3=4Cycle	; 7 Cycles */
                                	/* 0x4=5Cycle, 0x5=6Cycle */
                                	/* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON2     (initROMEndPtr2+initROMBasePtr2+rTacc2+rTpa2+PMC2)
#define workROMCON2     (workROMEndPtr2+workROMBasePtr2+rTacc2+rTpa2+PMC2)

/************************************************************************/
/* -> ROMCON3 :ROM Bank3 Control register				*/
/* 		Offset Address = 0x3020	Reset Value    = 0x00000060)	*/
/************************************************************************/
#define initROMBasePtr3   (0x060<<10)   /*=0x0600000 ;  6 */
#define initROMEndPtr3    (0x080<<20)   /*=0x0800000 ;  8 */
#define workROMBasePtr3   (0x260<<10)   /*=0x2600000 ; 38 */
#define workROMEndPtr3    (0x280<<20)   /*=0x2800000 ; 40 */
/* Page Mode Configuration */
#define PMC3                      0x0   /* 0x0=Normal ROM, 0x1=4Word Page */
                              	        /* 0x2=8Word Page, 0x3=16Word Page */
/* Page Access Time */
#define rTpa3                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
/* Programmable access cycle */
#define rTacc3               (0x0<<4)   /* 0x0=Disable, 0x1=2Cycle ; Disable */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
                                        /* 0x4=5Cycle, 0x5=6Cycle */
                                        /* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON3     (initROMEndPtr3+initROMBasePtr3+rTacc3+rTpa3+PMC3)
#define workROMCON3     (workROMEndPtr3+workROMBasePtr3+rTacc3+rTpa3+PMC3)

/************************************************************************/
/* -> ROMCON4 :ROM Bank4 Control register				*/
/*		Offset Address = 0x3024 Reset  Value   = 0x00000060  	*/
/************************************************************************/
#define initROMBasePtr4   (0x080<<10)   /*=0x0800000 ;  8 ; Disable */
#define initROMEndPtr4    (0x0A0<<20)   /*=0x0A00000 ; 10 */	
#define workROMBasePtr4   (0x280<<10)   /*=0x2800000 ; 40 */
#define workROMEndPtr4    (0x2A0<<20)   /*=0x2A00000 ; 42 */	
/* Page Mode Configuration */ 
#define PMC4                    (0x0)   /* 0x0=Normal ROM, 0x1=4Word Page */
                                        /* 0x2=8Word Page, 0x3=16Word Page */
/* Page Access Time */
#define rTpa4                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle */
                                        /* 0x2=3Cycle, 0x3=4Cycle */ 
/* Programmable access cycle */
#define rTacc4               (0x0<<4)   /* 0x0=Disable, 0x1=2Cycle ; Disable */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
                                        /* 0x4=5Cycle, 0x5=6Cycle */
                                        /* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON4     (initROMEndPtr4+initROMBasePtr4+rTacc4+rTpa4+PMC4)
#define workROMCON4     (workROMEndPtr4+workROMBasePtr4+rTacc4+rTpa4+PMC4)

/************************************************************************/
/* -> ROMCON5 :ROM Bank5 Control register 				*/
/*		Offset Address = 0x3028	Reset Value = 0x00000060 	*/
/************************************************************************/
#define initROMBasePtr5   (0x0A0<<10)   /* =0x0A00000 ; 10 ; Disable */
#define initROMEndPtr5    (0x0C0<<20)   /* =0x0C00000 ; 12 */
#define workROMBasePtr5   (0x2A0<<10)   /* =0x2A00000 ; 42 */
#define workROMEndPtr5    (0x2C0<<20)   /* =0x2C00000 ; 44 */

/* Page Mode Configuration */
#define PMC5                    (0x0)   /* 0x0=Normal ROM, 0x1=4Word Page */ 
                                        /* 0x2=8Word Page, 0x3=16Word Page */
/* Page Access Time */
#define rTpa5                (0x0<<2)   /* 0x0=5Cycle, 0x1=2Cycle */
                                        /* 0x2=3Cycle, 0x3=4Cycle */ 
/* Programmable access cycle */
#define rTacc5               (0x0<<4)   /* 0x0=Disable, 0x1=2Cycle ; Disable */
                                        /* 0x2=3Cycle, 0x3=4Cycle */
                                        /* 0x4=5Cycle, 0x5=6Cycle */
                                        /* 0x6=7Cycle, 0x7=Reserved */

#define initROMCON5     (initROMEndPtr5+initROMBasePtr5+rTacc5+rTpa5+PMC5)
#define workROMCON5     (workROMEndPtr5+workROMBasePtr5+rTacc5+rTpa5+PMC5)

/************************************************************************/
/* -> DRAMCON0 :RAM Bank0 control register     				*/
/* 		Offset      = 0x302C Reset Value = 0x0000 0000		*/
/*									*/
/*[0]		EDO mode 						*/
/*		0 = Normal DRAM (Fast page mode DRAM)			*/
/*		1 = EDO DRAM						*/
/*									*/
/*[2:1]	CAS strobe time	(tCS)						*/
/*		00 = 1 cycle	01 = 2 cycles				*/
/*		10 = 3 cycles	11 = 4 cycles				*/
/*									*/
/*[3:3]	CAS pre-charge time (tCP)					*/
/*		0 = 1 cycle	1 = 2 cycles				*/
/*									*/
/*[6:4]	Reserved							*/
/*									*/
/*[7]		RAS to CAS delay (tRC)					*/
/*		0 = 1 cycle	1 = 2 cycle 				*/
/*									*/
/*[9:8]	RAS pre-charge time (tRP)					*/
/*		00 = 1 cycle	01 = 2 cycles				*/
/*		10 = 3 cycles	11 = 4 cycles				*/
/*									*/
/*[19:10]	DRAM bank base pointer 					*/
/*[29:20]	DRAM bank next pointer 					*/
/*									*/
/*[31:30]	Number of column address bits in DRAM bank 		*/
/*		00 = 8 bits	01 = 9 bits				*/
/*		10 = 10 bits	11 = 11 bits				*/
/************************************************************************/
/* EDO Mode [0] (note: in SDRAM mode, this bit affect SDRAM cycle)*/
#define EDO_Mode0                   1   /*(EDO)0=Normal (Fast page mode DRAM), 1=EDO DRAM */
/* CAS strobe time */
#define CasStrobeTime0              1   /*(Tcs)0=1cycle,1=2cycles,2=3cycles,3=4cycle*/
/* CAR pre-charge time (note: in SDRAM mode, this bit affect SDRAM cycle)*/
#define CasPrechargeTime0           0   /*(Tcp)0=1cycle,1=2cycle */
/* Reserved */
#define DRAMCON0Reserved            1   /* Must be set to 1 */
/* RAS to CAS delay */
#define RAS2CASDelay0               0   /*(Trc)0=1cycle,1=2cycle */
/* RAS pre-charge time */
#define RASPrechargeTime0           2   /*(Trp)0=1cycle ~ 3=4clcyle */
/* DRAM bank base pointer */
#define initDRAMBasePtr0  (0x100<<10)	/*=0x1000000 ; 16 */
/* DRAM bank Next pointer */
#define initDRAMEndPtr0   (0x200<<20)	/*=0x2000000 ; 32 */
#define workDRAMBasePtr0  (0x000<<10)	/*=0x0000000 ;  0 */ 
#define workDRAMEndPtr0   (0x100<<20) 	/*=0x1000000 ; 16 */
#define NoColumnAddr0               2   /* 0=8bit,1=9bit,2=10bit,3=11bits */
/* NOTE : in SDRAM mode, this bit affect SDRAM cycle. */
/*        tCS value[1]: 0 = 1 cycle, 1=2 cycle */

#define Tcs0  (CasStrobeTime0   << 1)	/* CAS strobe time */		
#define Tcp0  (CasPrechargeTime0<< 3)	/* CAS pre-charge time */
#define dumy0 (DRAMCON0Reserved << 4) 	/* dummy cycle */
#define Trc0  (RAS2CASDelay0    << 7)	/* RAS to CAS delay */
#define Trp0  (RASPrechargeTime0<< 8)	/* RAS pre-charge time */
#define CAN0  (NoColumnAddr0    <<30)	/* Number of cilumn address bits */

#define initDRAMCON0 (CAN0+initDRAMEndPtr0+initDRAMBasePtr0+Trp0+Trc0+Tcp0+Tcs0+dumy0+EDO_Mode0)
#define workDRAMCON0 (CAN0+workDRAMEndPtr0+workDRAMBasePtr0+Trp0+Trc0+Tcp0+Tcs0+dumy0+EDO_Mode0)
/*----------------------------------------------------------------------*/
/* SDRAM Bank 0 settings */
/* [7] Ras to CAS Delay */
#define SRAS2CASDelay0  	    0   /* (Trc)0=1cycle,1=2cycle */
/* [9:8] RAS pre-charge time 00=1 cycle, 01=2 cycles, 10=3 cycles, 11=4 cycles*/
#define SRASPrechargeTime0 	    1   /* (Trp)0=1cycle ~ 3=4clcyle */
/* [31:30] Number of column address bits in DRAM bank 0*/
#define SNoColumnAddr0    	    0   /* 0=8bit,1=9bit,2=10bit,3=11bits */
/* HY57V651620B */
/* - Row Address: RA0-RA11, Column Address: CA0-CA7 */
/* - Programmable Burst Length and Burst Type*/
/* 	- 1,2,4,8 or Full page for Sequential Burst */
/* 	- 1,2,3,4 for Interleave Burst */
/* - Programmable CAS Latensy : 2,3 Clocks */

#define STrc0 (SRAS2CASDelay0    <<7)	/* RAS to CAS Delay */
#define STrp0 (SRASPrechargeTime0<<8)	/* RAS pre-charge time */
#define SCAN0 (SNoColumnAddr0  << 30)	/* Number of column address bits */ 

#define initSDRAMCON0 (SCAN0+initDRAMEndPtr0+initDRAMBasePtr0+STrp0+STrc0+dumy0)
#define workSDRAMCON0 (SCAN0+workDRAMEndPtr0+workDRAMBasePtr0+STrp0+STrc0+dumy0)
/************************************************************************/
/* -> DRAMCON1:RAM Bank1 control register 				*/
/* 		Offset      = 0x3030 Reset Value = 0x00000000		*/
/************************************************************************/
/* [0] EDO mode: 0=Normal EDO (Fast page mode DRAM), 1=EDO DRAM */
#define EDO_Mode1                   1   /* (EDO)0=Normal, 1=EDO DRAM */
/* [2:1] CAS strobe time: 00=1 cycle, 01=2 cycles, 10=3 cycles, 11=4 cycles */
#define CasStrobeTime1              1   /* (Tcs)0=1cycle ~ 3=4cycle */
/* [3:3] CAS pre-charge time: 0=1 cycle, 1=2 cycles*/
#define CasPrechargeTime1  	    0   /* (Tcp)0=1cycle,1=2cycle */
/* [6:4] This bits default value is 000. But, you must set to 001 */
#define DRAMCON1Reserved   	    1   /* Must be set to 1 */
/* [7]   RAS to CAS delay : 0=1cycle. 1=2 cycles */
#define RAS2CASDelay1               0   /* (Trc)0=1cycle,1=2cycle */
/* [9:8] RAS pre-charge time : 00=1 cycle, 01=2 cycles, 10=3 cycles, 11=4 cycles*/
#define RASPrechargeTime1     	    0   /* (Trp)0=1cycle ~ 3=4clcyle */
#define initDRAMBasePtr1  (0x200<<10)   /* [19:10] DRAM bank 1 base pointer */ 
#define initDRAMEndPtr1   (0x240<<20)   /* [29:20] DRAM bank 1 Next pointer */
#define workDRAMBasePtr1  (0x100<<10) 
#define workDRAMEndPtr1   (0x140<<20) 
/* [31:30] Number of column address bits in DRAM bank 1 */
#define NoColumnAddr1              2    /* 0=8bit,1=9bit,2=10bit,3=11bits */
/*----------------------------------------------------------------------*/
#define Tcs1   (CasStrobeTime1   <<1)	/* CAS strobe time */
#define Tcp1   (CasPrechargeTime1<<3)	/* CAS pre-charge time */
#define dumy1  (DRAMCON1Reserved <<4) 	/* dummy cycle */
#define Trc1   (RAS2CASDelay1    <<7)	/* RAS to CAS delay */ 
#define Trp1   (RASPrechargeTime1<<8)	/* RAS pre-charge time */ 
#define CAN1   (NoColumnAddr    <<30)	/* Number of column address bits in DRAM bank 1 */

#define initDRAMCON1 (CAN1+initDRAMEndPtr1+initDRAMBasePtr1+Trp1+Trc1+Tcp1+Tcs1+dumy1+EDO_Mode1)
#define workDRAMCON1 (CAN1+workDRAMEndPtr1+workDRAMBasePtr1+Trp1+Trc1+Tcp1+Tcs1+dumy1+EDO_Mode1)    
/*----------------------------------------------------------------------*/
/* [7] RAS to CAS delay */
#define SRAS2CASDelay1              1   /* (Trc)0=1cycle,1=2cycle */
/* [9:8] RAS pre-charge time */
#define SRASPrechargeTime1          1   /* (Trp)0=1cycle ~ 3=4clcyle */
/* [31:30] Number of column address bits in DRAM bank 1*/
#define SNoColumnAddr1     	    0   /* 0=8bit,1=9bit,2=10bit,3=11bits */

#define SCAN1 (SNoColumnAddr1   <<30)
#define STrc1 (SRAS2CASDelay1    <<7)
#define STrp1 (SRASPrechargeTime1<<8)

#define initSDRAMCON1 (SCAN1+initDRAMEndPtr1+initDRAMBasePtr1+STrp1+STrc1)
#define workSDRAMCON1 (SCAN1+workDRAMEndPtr1+workDRAMBasePtr1+STrp1+STrc1)  		
/************************************************************************/
/* -> DRAMCON2:RAM Bank2 control register 				*/
/* 		Offset = 0x3034 					*/
/************************************************************************/
/*[0] EDO Mode : 0=Normal DRAM (Fast page mode DRAM), 1=EDO DRAM */
#define EDO_Mode2           	    0   /* (EDO)0=Normal, 1=EDO DRAM */
/*[2:1] CAS strobe time : 00=1 cycle, 01=2 cycles, 10=3 cycles, 11=4 cycles */
#define CasStrobeTime2    	    1   /* (Tcs)0=1cycle ~ 3=4cycle */
/*[3] CAS precharge time*/
#define CasPrechargeTime2  	    0   /* (Tcp)0=1cycle,1=2cycle */
/*[6:4]Reserved : This bits default value is 000. But, you must set to 001  */
#define DRAMCON2Reserved   	    1   /* Must be set to 1 */
/*[7] RAS to CAS delay */
#define RAS2CASDelay2    	    0   /* (Trc)0=1cycle,1=2cycle */
/*[9:8] RAS pre-charge time */
#define RASPrechargeTime2  	    0   /* (Trp)0=1cycle ~ 3=4clcyle */
/*[19:10] DRAM bank 2 base pointer */
#define initDRAMBasePtr2  (0x240<<10)
/*[29:20] DRAM bank 2 Next pointer */
#define initDRAMEndPtr2   (0x280<<20)         	
#define workDRAMBasePtr2  (0x140<<10)/*=0x14000000 */
#define workDRAMEndPtr2   (0x180<<20)/*=0x18000000 */
/*[31:30] Number of column address bits in DRAM bank 2 */
#define NoColumnAddr2  	  	    2   /* 0=8bit,1=9bit,2=10bit,3=11bits */
/*----------------------------------------------------------------------*/
#define Tcs2   (CasStrobeTime2   <<1)   /* CAS strobe time */
#define Tcp2   (CasPrechargeTime2<<3)   /* CAS precharge time */ 
#define dumy2  (DRAMCON2Reserved <<4)   /* dummy cycle */
#define Trc2   (RAS2CASDelay2    <<7)   /* RAS to CAS delay */ 
#define Trp2   (RASPrechargeTime2<<8)   /* RAS precharge time */
#define CAN2   (NoColumnAddr2   <<30)   /* Number of column address bits */ 

#define initDRAMCON2	(CAN2+initDRAMEndPtr2+initDRAMBasePtr2+Trp2+Trc2+Tcp2+Tcs2+dumy2+EDO_Mode2)
#define workDRAMCON2	(CAN2+workDRAMEndPtr2+workDRAMBasePtr2+Trp2+Trc2+Tcp2+Tcs2+dumy2+EDO_Mode2)
/*----------------------------------------------------------------------*/
/* [7] RAS to CAS delay */
#define SRAS2CASDelay2              1   /* (Trc)0=1cycle,1=2cycle */
/* [9:8] RAS precharge time */
#define SRASPrechargeTime2  	    1   /* (Trp)0=1cycle ~ 3=4clcyle */
/* [31:30]Number of column address bits */
#define SNoColumnAddr2              0   /* 0=8bit,1=9bit,2=10bit,3=11bits */

#define SCAN2 (SNoColumnAddr2   <<30)
#define STrc2 (SRAS2CASDelay2    <<7)
#define STrp2 (SRASPrechargeTime2<<8)

#define initSDRAMCON2	(SCAN2+initDRAMEndPtr2+initDRAMBasePtr2+STrp2+STrc2)
#define workSDRAMCON2	(SCAN2+workDRAMEndPtr2+workDRAMBasePtr2+STrp2+STrc2)
/*----------------------------------------------------------------------*/

/************************************************************************/
/* -> DRAMCON3:RAM Bank3 control register 				*/
/* 		Offset = 0x3038 					*/
/************************************************************************/
/*----------------------------------------------------------------------*/ 
/*[0] EDO Mode: 0=Normal DRAM (Fast page mode DRAM), 1=EDO DRAM */
#define EDO_Mode3          	    0   /* (EDO)0=Normal, 1=EDO DRAM */
/*[2:1] CAS strobe time : 00=1 cycle,01=2 cycles, 10=3 Cycles, 11=4 cycles*/
#define CasStrobeTime3     	    1   /* (Tcs)0=1cycle ~ 3=4cycle */
/*[3]CAS pre-charge time*/
#define CasPrechargeTime3    	    0   /* (Tcp)0=1cycle,1=2cycle */
/*[6:4] Rseserved. This value is set 000. But, you must set to 001*/
#define DRAMCON3Reserved   	    1   /* Must be set to 1 */
/*[7] RAS to CAS delay */
#define RAS2CASDelay3               0   /* (Trc)0=1cycle,1=2cycle */
/*[9:8] RAS pre-charge time : 00=1 cycle, 01=2 cycles, 10=3 cycles, 11=4 cycles */
#define RASPrechargeTime3  	    0   /* (Trp)0=1cycle ~ 3=4clcyle */
#define initDRAMBasePtr3  (0x280<<10)   /*[19:10]DRAM bank 3 base pointer */
#define initDRAMEndPtr3   (0x2C0<<20)   /*[29:20] DRAM bank 3 Next pointer*/
#define workDRAMBasePtr3  (0x180<<10)
#define workDRAMEndPtr3   (0x1C0<<20)  

/*[31:30] Number of column address bits in DRAM bank 3*/
#define NoColumnAddr3        	    2   /* 0=8bit,1=9bit,2=10bit,3=11bits */
/*----------------------------------------------------------------------*/
#define Tcs3      (CasStrobeTime3<<1)
#define Tcp3   (CasPrechargeTime3<<3)
#define dumy3   (DRAMCON3Reserved<<4)	/* dummy cycle */
#define Trc3       (RAS2CASDelay3<<7)
#define Trp3   (RASPrechargeTime3<<8)
#define CAN3      (NoColumnAddr3<<30)
/* */
#define initDRAMCON3 (CAN3+initDRAMEndPtr3+initDRAMBasePtr3+Trp3+Trc3+Tcp3+Tcs3+dumy3+EDO_Mode3)
#define workDRAMCON3 (CAN3+workDRAMEndPtr3+workDRAMBasePtr3+Trp3+Trc3+Tcp3+Tcs3+dumy3+EDO_Mode3)

/*----------------------------------------------------------------------*/
/*[7] RAS to CAS delay */
#define SRAS2CASDelay3              1   /* (Trc)0=1cycle,1=2cycle */
/*[9:8] RAS pre-charge time*/
#define SRASPrechargeTime3          1   /* (Trp)0=1cycle ~ 3=4clcyle */
/*[31:30] Number of column address bits in DRAM bank 3*/
#define SNoColumnAddr3              0   /* 0=8bit,1=9bit,2=10bit,3=11bits */

#define SCAN3    (SNoColumnAddr3<<30)
#define STrc3     (SRAS2CASDelay3<<7)
#define STrp3 (SRASPrechargeTime3<<8)
/* */
#define initSDRAMCON3 (SCAN3+initDRAMEndPtr3+initDRAMBasePtr3+STrp3+STrc3)
#define workSDRAMCON3 (SCAN3+workDRAMEndPtr3+workDRAMBasePtr3+STrp3+STrc3)
/*----------------------------------------------------------------------*/

/************************************************************************/
/* ->REFEXTCON:External I/O & Memory Refresh cycle Control Register 	*/
/* 		Offset = 0x303C 					*/
/************************************************************************/
/***********************************************************************/
/*[9-0] 	External I/O bank 0 base pointer (base address)		*/
/*[15] 	Validity of special register field (VSF)			*/
/*		0 = No accessible to memory bank			*/
/*		1 = Accessible to memory bank				*/
/*[16] 	Refresh enable (REN)						*/
/*		0 = Disable DRAM refresh				*/
/*		1 = Enable DRAM refresh					*/
/*[19:17] 	CAS hold time (tCHR)					*/
/*		ROW Cycle Time (tRC) (note 1)				*/
/*		000=1 cycle	001=2 cycles				*/
/*		010=3 cycles	011=4 cycles				*/
/*		100=5 cycles	101=Not Used (6 cycles)			*/
/*	110=Not used	111=Not Used 					*/
/*									*/
/*		NOTE 1: In EDO/normal DRAM mode, CAS hold time		*/
/*		can bee programmed upto 5 cycles.			*/
/*		But in SDRAM mode, this bit fields 			*/
/*		function are defined as ROW Cycle			*/
/*		Time (tRC) and can be programmed up			*/
/*		to 6 cycles.						*/
/*[20] 	CAS setup time (tCSR) (note 2)					*/
/*		0=1 cycle						*/
/*		1=2 cycles						*/
/*     									*/
/*      	NOTE 2: In SDRAM mode, this bit field is 		*/
/*      		reserved.					*/
/*[31:21] 	Refresh count value (duration) 				*/
/*		The refresh period is calculated as 			*/
/* 		(2**11 - Value + 1) fMCLK				*/
/************************************************************************/

#define RefCycle                   16   /* Unit [us], 1k refresh 16ms */
#define CASSetupTime                0   /* 0=1cycle, 1=2cycle */
#define CASHoldTime                 0   /* 0=1cycle, 1=2cycle, 2=3cycle, */
                        	        /* 3=4cycle, 4=5cycle */

#define RefCycleValue   ((2048+1-(RefCycle*fMCLK)) << 21)

#define Tcsr     (CASSetupTime << 20)   /* 1 cycle */
#define Tcs      (CASHoldTime  << 17)   /* 1 cycle */
#define ExtIOBase             0x18360   /* Refresh enable, VSF=1 */

/*    1    8    3    6    0 */
/* 0001 1000 0011 0110 0000 */
/*    6 5432 1098 7654 3210 */
/* External Base 	= 0x360 => 0x360 0000  */
/* [15] VSF 		= 1 */
/* [16] Refresh enable 	= 1 */
#define rREFEXTCON       (RefCycleValue+Tcsr+Tcs+ExtIOBase)
/*----------------------------------------------------------------------*/
#define SRefCycle                  16   /* Unit [us], 4k refresh 64ms */
/* ROW Cycle time */
#define ROWcycleTime      	    4   /* 0=1cycle, 1=2cycle, 2=3cycle, */
                        		/* 3=4cycle, 4=5cycle, */
/* */
#define SRefCycleValue  ((2048+1-(SRefCycle*fMCLK))<<21)
#define STrc            (ROWcycleTime<<17) 
/* */   
#define rSREFEXTCON     (SRefCycleValue+STrc+ExtIOBase)
/*----------------------------------------------------------------------*/

/************************************************************************/
/* KS32C50100 SPECIAL REGISTERS                                         */
/************************************************************************/

#define ASIC_BASE  	    0x3ff0000

/* I/O Port Interface */
#define IOPMOD	   (ASIC_BASE+0x5000)	/* I/O port mode register */
/*Reset value=0x00000000 (All I/O port pins - inputs)				*/
/*[ 0] I/O port mode bit for port 0						*/
/*[17] I/O port mode bit for port 0						*/
/*	0=Input									*/
/*	1=Output								*/
#define IOPCON     (ASIC_BASE+0x5004)	/* I/O port control register 	*/
#define IOPDATA    (ASIC_BASE+0x5008)	/* I/O port data register 	*/

/*  UART SPECIAL REGISTERS 0,1  */
#define UARTLCON0  (ASIC_BASE+0xD000)	/* UART0 Line Control register */
#define UARTCONT0  (ASIC_BASE+0xD004)	/* UART0 Control register */
#define UARTSTAT0  (ASIC_BASE+0xD008)	/* UART0 status register */
#define UARTTXH0   (ASIC_BASE+0xD00C)	/* UART0 transmit buffer register */
#define UARTRXB0   (ASIC_BASE+0xD010)	/* UART0 receive buffer register */
#define UARTBRD0   (ASIC_BASE+0xD014)	/* UART0 baud rate divisor register */

#define UARTLCON1  (ASIC_BASE+0xE000)	/* UART 1 Line Control register */
#define UARTCONT1  (ASIC_BASE+0xE004)	/* UART 1 Control register */
#define UARTSTAT1  (ASIC_BASE+0xE008)	/* UART 1 Status register */
#define UARTTXH1   (ASIC_BASE+0xE00C)	/* UART 1 transmit buffer register */
#define UARTRXB1   (ASIC_BASE+0xE010)	/* UART 1 receiver buffer register */
#define UARTBRD1   (ASIC_BASE+0xE014)	/* UART 1 baud rate divisor */

#define UCON0  	   (ASIC_BASE+0xD000)	/* UART0 control register s3C4530 */
#define USTAT0	   (ASIC_BASE+0xD004)	/* UART0 status register s3C4530 */
#define UCON1  	   (ASIC_BASE+0xE000)	/* UART1 control register s3C4530  */
#define USTAT1	   (ASIC_BASE+0xE004)	/* UART1 status register s3C4530 */
/************************************************************************/
